//////////////////////////////////////////////////////////////////////////////
//	(c) Diamond Systems Corporation 2004
//	
//	http://www.diamondsystems.com
//
//	(c) Copyright 2004 Diamond Systems Corporation. Use of this source code
//	is subject to the terms of Diamond Systems' Software License Agreement.
//	Diamond Systems provides no warranty of proper performance if this
//	source code is modified.
//
//	By KL
//	Updated 1/15/04
///////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include "dscud.h"
#include "adtests.h"

/*=============================================
  A/D Quick Diagnostic Test. Samples and scans
  calmux channels then compares values
  to reference voltages.
=============================================*/

BYTE ADTest(BYTE boardType, DSCB dscb)
{
	ERRPARAMS errorParams;  //structure with error information
	FLOAT refs[5];          //array for sotring reference voltages
	char errorString[500];  //error string
	
	int i;
	int result;
	int tolerance = 0;
	int option;            //option 0 is sample, option 1 is scan
	int num_chan;          //number of channels
	long diff;             //difference between expected and sampled
	int error;             //number of errors
	int refIndex;          //reference voltage array index
	DSCADSCAN dscadscan;   //structure for A/D scan
	DSCSAMPLE * samples;   //pointer to array for A/D Scan
	DSCSAMPLE sample;      //variable to contain sample value
	DSCSAMPLE adcode;
	DSCADSETTINGS dscadsettings;
	BYTE admode = 8;      //A/D mode which consists of range, polarity, and gain.

	getReference(refs);   //obtain the reference voltages

	clrscr();    //clear screen
	printf("\n");
	printf("    *================== A/D 'QUICK' Diagnostic Test ==================*\n");
	fileprint("\n    *================== A/D 'QUICK' Diagnostic Test ==================*\n");
		
	num_chan = 5; // number of reference channels

	//setting the tolerance for different boards
	if( boardType == DSC_DMMAT)
		tolerance = 5;
	else if((boardType == DSC_DMM16AT) || (boardType == DSC_DMM48))
		tolerance = 10;
	else if(boardType == DSC_HERCEBX)
	{
		tolerance = 10;
		num_chan = 4;
	}
	else if(boardType == DSC_DMM32)
		tolerance = 120;

	printf("\n\t");
	printf("Testing calmux channels 0-%d\n", num_chan-1);

	//setting the A/D mode, such as range, gain, polarity, etc.
	dscadsettings.range    = 1;
	dscadsettings.polarity = 0;
	dscadsettings.gain     = 0;
	dscadsettings.load_cal = 1;
	dscadsettings.current_channel = 0;
	if(boardType == DSC_DMM16AT || boardType == DSC_DMMAT || boardType == DSC_DMM32)
		dscadsettings.scan_interval = SCAN_INTERVAL_10;
	else 
		dscadsettings.scan_interval = SCAN_INTERVAL_5;
	dscadsettings.addiff = 0;

	if(boardType == DSC_DMM48)
	{
		//setting A/D mode for DMM-48-AT
		admode = getDMM48Mode();
		dscadsettings.range = (admode>>3)&0x1;
		dscadsettings.polarity = (admode>>2)&0x1;
		dscadsettings.gain = (admode&0x3);
	}

	//performing A/D settings
	if((result = dscADSetSettings(dscb, &dscadsettings)) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		sprintf(errorString, "Error in dscADSetSettings, DSC Error: #%d.\n\t%s", result, errorParams.errstring);
		errormessage(errorString);
		return 4;
	}

	//enable calmux to sample calmux channels
	dscSetCalMux(dscb, 1);

	error = 0;
	option = 0;

	for ( option = 0; option < 2; option ++)
	{
		if( option == 0)
		{
			// AD Scan Test
			if( boardType == DSC_DMM16AT || boardType == DSC_DMMAT || boardType == DSC_DMM32)
			{
				printf("\n\tA/D Scan Test, 10 micro seconds scan interval\n");
				fileprint("\n\tA/D Scan Test, 10 micro seconds scan interval\n");
			}
			else
			{
				printf("\n\tA/D Scan Test, 5 micro seconds scan interval\n");
				fileprint("\n\tA/D Scan Test, 5 micro seconds scan interval\n");
			}

			dscadscan.low_channel = 0;
			dscadscan.high_channel = num_chan - 1;
			dscadscan.gain = dscadsettings.gain;

			//allocating space for array
			samples = (DSCSAMPLE*) malloc(sizeof(DSCSAMPLE) * num_chan);
			memset(samples, 0, sizeof(DSCSAMPLE) * num_chan);

			error = 0;
			result = dscADScan(dscb, &dscadscan, samples);
		}
		else if( option==1 )
		{
			// AD Sample Test
			printf("\n\t");
			printf("A/D Sample Test\n");
			fileprint( "\n\tA/D Sample Test\n");
		}

		if( result != DE_NONE)
		{
			dscGetLastError(&errorParams);
			sprintf(errorString, "Error in dscADScan, DSC Error: #%d, %s\n", result, errorParams.errstring);
			errormessage(errorString);
			free(samples);
			dscSetCalMux(dscb, 0);
			return 3;
		}

		//printing things out, setting up the table header
		printf("\t");
		printf("Calmux ch   Expected   Sampled   Difference\n");
		fileprint("\tCalmux ch   Expected   Sampled   Difference\n");

		for ( i = 0; i < num_chan; i++)
		{
			if( option==1) //A/D sample option
			{
				//must change channels for each sample
				result = dscADSetChannel(dscb, i, i);
				if( result != DE_NONE)
				{
					dscGetLastError(&errorParams);
					sprintf(errorString, "Error in dscADSetChannel, DSC Error #%d\n\t%s", result, errorParams.errstring);
					errormessage(errorString);
					dscSetCalMux(dscb, 0);
					free(samples);
					return 2;
				}

				result = dscADSample(dscb, &sample);
				if( result != DE_NONE)
				{
					dscGetLastError(&errorParams);
					sprintf(errorString, "Error in dscADSample, DSC Error: #%d\n\t%s", result, errorParams.errstring);
					errormessage(errorString);
					dscSetCalMux(dscb, 0);
					free(samples);
					return 1;
				}
			}

			//obtaining the expected value and converting the voltage to A/D code
			//expected value is stored reference votlage
			refIndex = (i%num_chan) + dscadscan.low_channel;
			adcode = voltToADCode(boardType, admode, refs[refIndex]);
			
			if ( option == 1)
				samples[i] = sample;
			
			diff = labs((long)adcode - (long)samples[i]);
			
			if( diff > tolerance)
				error ++;

			printf("\t");
			printf("       %2d   %8d   %7d       %6ld\n", i+dscadscan.low_channel, adcode, samples[i], diff);
			sprintf(errorString, "\t       %2d   %8d   %7d       %6ld\n", i+dscadscan.low_channel, adcode, samples[i], diff);
			fileprint(errorString);
		}
	}

	free(samples);

	if( error > 0)
	{
		printf("\n\tA/D 'QUICK' Test Failed, values exceeded tolerance of %d LSB\n", tolerance);
		sprintf(errorString, "\n\tA/D Test Failed, values exceeded tolerance of %d LSB\n", tolerance);
		fileprint(errorString);
	}
	else
	{
		printf("\n\tA/D Test Passed\n");
		fileprint("\n\tA/D Test Passed\n");
		
	}

	dscSetCalMux(dscb, 0);
	endScreen();
	return 0;

}  //end ADTest


/*=============================================
  A/D User Test. Allows users to setup their 
  own A/D sample/scan settings.
=============================================*/

BYTE ADUserTest(BYTE boardType, DSCB dscb)
{
	ERRPARAMS errorParams;  //structure with error information
	char errorString[500];  //error string

	BYTE result, admode;
	int i;
	int num_chan;          //number of channels
	int option;            //option 0 is sample, 1 is scan
	int temp;              
	int scaninterval;      //scan interval for A/D scans
	int highchan=0;        //high channel
	int lowchan=0;         //low channel
	int sleeptime = 750;   //sleep time in milliseconds
	int times = 0;
	float voltage;
	DSCSAMPLE sample;      //variable to contain sample value
	DSCSAMPLE * samples;   //pointer to array for A/D Scan
	DSCADSCAN dscadscan;   //structure for A/D scan
	DSCADSETTINGS dscadsettings;  //A/D settings structure.

	clrscr();

	printf("\n");
	printf("    *================== A/D User Diagnostic Test ==================*\n");
	fileprint("\n    *================== A/D User Diagnostic Test ==================*\n");

	printf("\n\t");
	printf("Enter (0) for sample, (1) for scan : ");
	scanf("%d", &option);

	//DMM48 has only 3 modes, 8, 9, 13
	if( boardType != DSC_DMM48)
	{
		if( boardType == DSC_HERCEBX)
			temp = RANGE_10;
		else
		{
			printf("\t");
			printf("Enter (0) for Range_5, 5V; (1) for RANGE_10, 10V : ");
			scanf("%d", &temp);
		}
		dscadsettings.range = (BYTE) temp;
		printf("\t");
		printf("Enter (0) for bipolar, (1) for unipolar : ");
		scanf("%d", &temp);
		dscadsettings.polarity = (BYTE)temp;
		printf("\t");
		printf("Enter gain (1, 2, 4, 8) : ");
		scanf("%d", &temp);

		if( temp == 1)
			temp = 0;
		else if( temp == 2)
			temp = 1;
		else if( temp == 4)
			temp = 2;
		else if (temp == 8)
			temp = 3;
		else 
			temp = 100;

		dscadsettings.gain = (BYTE)temp;
		admode = (dscadsettings.range << 3)|(dscadsettings.polarity<<2)|(dscadsettings.gain);
	}
	else{
		printf("\t");
		printf("Valid Modes are 8 (+-10V), 9 (+-5V), 13 (0-5V)\n");
		printf("\t");
		printf("Enter the Mode (8, 9, 13) : ");
		scanf("%d", &temp);
		dscadsettings.range = RANGE_10;
		dscadsettings.polarity = (temp >> 2) & 0x01;
		dscadsettings.gain = (temp & 0x03);

		admode = temp;
	}

	if( boardType == DSC_HERCEBX)
	{
		printf("\t");
		printf("Enter (0) for single-ended, (1) for differential : ");
		scanf("%d", &temp);
		dscadsettings.addiff = temp;
	}

	//setting up A/D scan/sample
	if( option == 1)
	{
		if( boardType != DSC_DMM16AT || boardType != DSC_DMMAT)
		{
			printf("\t");
			printf("Enter (0) for scan interval 10 or 9 micro sec, (1) for 5 micro sec : ");
			scanf("%d", &scaninterval);

			if( boardType == DSC_DMM32)
				dscadsettings.scan_interval = (scaninterval == 0)?SCAN_INTERVAL_10:SCAN_INTERVAL_5;
			else
				dscadsettings.scan_interval = (scaninterval == 0)?SCAN_INTERVAL_9:SCAN_INTERVAL_5;
		}
		else 
			dscadsettings.scan_interval = SCAN_INTERVAL_5;
		
		printf("\t");
		printf("Enter low channel to scan : ");
		scanf("%d", &lowchan);
		printf("\t");
		printf("Enter high channel to scan : ");
		scanf("%d", &highchan);
		num_chan = highchan - lowchan + 1;
		dscadsettings.current_channel = (BYTE)lowchan;

		sleeptime = num_chan/3.0 * 750 + 750;
	}
	else
	{
		printf("\t");
		printf("Enter channel to sample : ");
		scanf("%d", &temp);
		dscadsettings.current_channel = (BYTE)temp;
		num_chan = 1;
	}

	dscadsettings.load_cal = (BYTE)1;

	getchar();

	if( ( result = dscADSetSettings( dscb, &dscadsettings ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		sprintf(errorString, "Error in dscADSetSettings, DSC Error: #%d\n\t%s", result, errorParams.errstring);
		errormessage(errorString);
		return 1;
	}

	printf("\n\tPress Enter to quit\n\n");
	printf("\t");
		
	if( option == 1 && num_chan > 15) 
	{
		printf("Channel   A/D Code   A/D Voltage    Channel   A/D Code   A/D Voltage\n");
		fileprint("\n\tChannel   A/D Code   A/D Voltage    Channel   A/D Code   A/D Voltage\n");	
	}
	else
	{
		printf("Channel   A/D Code   A/D Voltage\n");
		fileprint("\n\tChannel   A/D Code   A/D Voltage\n");
	}

	do
	{
		if(times > 8 )
		{
			printf("\n\tPress Enter to quit\n\n");
			printf("\t");

			if( option == 1 && (highchan - lowchan + 1) > 15) 
			{
				printf("Channel   A/D Code   A/D Voltage    Channel   A/D Code   A/D Voltage\n");
				fileprint("\n\tChannel   A/D Code   A/D Voltage    Channel   A/D Code   A/D Voltage\n");	
			}
			else
			{
				printf("Channel   A/D Code   A/D Voltage\n");
				fileprint("\n\tChannel   A/D Code   A/D Voltage\n");
			}
			times = 0;
		}


		if( option == 0)
		{
			//AD Sample User Test
			result = dscADSample(dscb, &sample);
			if( result != DE_NONE)
			{
				dscGetLastError(&errorParams);
				sprintf(errorString, "Error in dscADSample, DSC Error: #%d\n\t%s", result, errorParams.errstring);
				errormessage(errorString);
				return 2;
			}
				
			voltage = ADCodeToVolt(boardType, admode, sample);
			
			printf("\t");
      		printf("%7d   %8d   %11.3f\n", dscadsettings.current_channel, sample, voltage);
			sprintf(errorString, "\t%7d   %8d   %11.3f\n", dscadsettings.current_channel, sample, voltage);
			fileprint(errorString);
			times ++;
		}
		else 
		{
			//A/D Scan User Test
			dscadscan.gain = dscadsettings.gain;
			dscadscan.low_channel = lowchan;
			dscadscan.high_channel = highchan;

			samples = (DSCSAMPLE*)malloc(sizeof(DSCSAMPLE) * num_chan);
			memset(samples, 0, sizeof(DSCSAMPLE) * num_chan);


			result = dscADScan(dscb, &dscadscan, samples);
			if( result != DE_NONE)
			{
				dscGetLastError(&errorParams);
				sprintf(errorString, "Error in dscADScan, DSC Error: #%d\n\t%s\n", result, errorParams.errstring);
				errormessage(errorString);
				free(samples);
				return 3;
			}

			for (i = 0; i < num_chan; i++)
			{
				
				voltage = ADCodeToVolt(boardType, admode, samples[i]);
		
				if(num_chan > 15) 
				{
					if(i%2 == 0)
					{
						printf("\t");
						fileprint("\t");
					}
					else
					{
						printf("    ");
						fileprint("    ");
					}
				}
				else
				{
					printf("\t");
					fileprint("\t");
				}
			
				printf("%7d   %8d   %11.3f", i+dscadscan.low_channel, samples[i], voltage);
				sprintf(errorString, "%7d   %8d   %11.3f", i+dscadscan.low_channel, samples[i], voltage);
				fileprint(errorString);
				times ++;

				if((i%2) == 1 || num_chan < 16)
				{
					printf("\n");
					fileprint("\n");
				}
			}
			printf("\n");
			fileprint("\n");

			free(samples);
		}

		dscSleep(sleeptime);
	}	while( !kbhit());
	
	getchar();
	return 0;

} //end ADUserTest


