//////////////////////////////////////////////////////////////////////////////
//	(c) Diamond Systems Corporation 2004
//	
//	http://www.diamondsystems.com
//
//	(c) Copyright 2004 Diamond Systems Corporation. Use of this source code
//	is subject to the terms of Diamond Systems' Software License Agreement.
//	Diamond Systems provides no warranty of proper performance if this
//	source code is modified.
//
//	By KL
//	Updated 1/15/04
///////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include "dscud.h"
#include "autocal.h"

/*=====================================================
	A/D Autocal Test. Performs A/D autocalibration test
======================================================*/

BYTE ADAutocal(BYTE boardType, DSCB dscb)
{
	ERRPARAMS errorParams;  //structure with error information
	char errorString[500];  //error string

	int i;
	BYTE result;
	DSCADCALPARAMS adcalparams;
	int min = 0;     //dmmat, 16at, 32at uses modes 0-15. Minimum mode
	int max = 16;    //and maximum mode for boards.

	clrscr();

	printf("\n");
	printf("    *==================== A/D Autocal Test ====================*\n");
	fileprint("\n    *==================== A/D Autocal Test ====================*\n");

	printf("\n\tTest may take 30 seconds or more to complete\n\tas each mode takes 2-3 seconds.\n");

	if( boardType == DSC_HERCEBX)
		max = 8;  //mode for Herc EBX
	else if ( boardType == DSC_DMM48)
	{
		//mode for DMM-48
		min = getDMM48Mode();
		max = min+1;
	}

	printf("\n\t");
	printf("If error is greater than 2, test failed\n\n");
	printf("\t");
	printf("Mode Calibrated   Offset Error   Gain Error   Pass/Fail\n");	
	fileprint("\n\tMode Calibrated   Offset Error   Gain Error   Pass/Fail\n");	

	for ( i = min; i < max; i++)
	{
		//A/D autocal
		adcalparams.adrange = i;
		adcalparams.boot_adrange = i;
		result = dscADAutoCal(dscb, &adcalparams);
		if( result == DE_INVALID_PARM)
			continue;
		else if( result != DE_NONE)
		{
			dscGetLastError(&errorParams);
			sprintf(errorString, "Error in dscADAutocal, DSC Error: #%d\n\t%s", result, errorParams.errstring);
			errormessage(errorString);
			return 1;
		}

		//Verify calibration result
		dscADCalVerify(dscb, &adcalparams);
		printf("\t");
		printf("             %2d      %9.2f    %9.2f        ", i, (float)adcalparams.ad_offset, (float)adcalparams.ad_gain);
		sprintf(errorString, "\t             %2d      %9.2f    %9.2f        ", i, (float)adcalparams.ad_offset, (float)adcalparams.ad_gain);
		fileprint(errorString);

		if( fabs(adcalparams.ad_gain) > 2 || fabs(adcalparams.ad_offset) > 2)
		{
			printf("FAIL\n");
			fileprint("FAIL\n");
		}
		else
		{
			printf("PASS\n");
			fileprint("PASS\n");
		}
	}

	endScreen();
	return 0;
} // ADAutocal


/*==================================================
	D/A Autocal Test. Performs D/A autocalibration 
===================================================*/

BYTE DAAutocal(BYTE boardType, DSCB dscb)
{
	ERRPARAMS errorParams;  //structure with error information
	char errorString[500];  //error string

	int intBuff;            //variable for scanf integers
	int error;              //number of errors
	FLOAT min, max;         //minimum and maximum D/A output range in voltage
	BYTE result;
	DSCDACALPARAMS dacalparams;
	
	min = 0;
	max = 0;
	
	clrscr();

	printf("\n");
	printf("    *==================== D/A Autocal Test ====================*\n");
	fileprint("\n    *==================== D/A Autocal Test ====================*\n");

	dacalparams.fixed = 1;
	dacalparams.darange = 0;

	if( boardType != DSC_HERCEBX)
	{
		printf("\n\tThis is a quick test of the D/A autocal and will only test\n");
		printf(  "\tand calibrate D/A mode fixed. To test programmable mode\n");
		printf(  "\tplease refer to the dscDAAutocal.exe sample program.\n");
	}
	
	if( boardType != DSC_HERCEBX && boardType != DSC_DMM48)
	{
		printf("\n\t");
		printf("Please set the D/A setting jumpers to fixed mode by jumpering\n");
		printf("\tthe 5 or 10 jumper and leave the P jumper out as shown below.\n");
		printf("\n");
		DAJumperMenu(boardType); //Jumper setting printout
		continueScreen();

		//Find the D/A voltage range.
		DAMinMaxVolt(boardType, &min, &max);

		printf("\tCalibrating for range %4.2fV to %4.2fV.\n\tIf range listed is incorrect there is an error with the board.\n", min, max);
		sprintf(errorString, "\tCalibrating for range %4.2fV to %4.2fV.\n\tIf range listed is incorrect there is an error with the board.\n", min, max);
		fileprint(errorString);

		if( boardType == DSC_DMMAT)
		{
			dacalparams.ch0ext = 0;
			dacalparams.ch0pol = 0;
			dacalparams.ch0prog = 0;
			dacalparams.ch1ext = 0;
			dacalparams.ch1pol = 0;
			dacalparams.ch1prog = 0;
		}
	}
	else if( boardType == DSC_HERCEBX)
	{
		printf("\n\t");
		printf("Enter the D/A polarity (0) for BIPOLAR or (1) for UNIPOLAR : ");
		scanf("%d", &intBuff);
		getchar();
	}
	else if( boardType == DSC_DMM48)
		dacalparams.darange = 4.096;
	
	printf("\n\t");
	printf("If error is greater than 2, test failed.\n\n");

	//D/A autocal
	result = dscDAAutoCal(dscb, &dacalparams);
	if( result != DE_NONE)
	{
		dscGetLastError(&errorParams);
		sprintf(errorString, "Error in dscDAAutocal, DSC Error: #%d\n\t%s", result, errorParams.errstring);
		errormessage(errorString);
		return 1;
	}

	//Verify calibration result of the polarity setting
	if( boardType == DSC_HERCEBX)
		dacalparams.polarity = (BYTE) intBuff;
	
	//Verify calibration result
	result = dscDACalVerify(dscb, &dacalparams);
	if( result != DE_NONE)
	{
		dscGetLastError(&errorParams);
		sprintf(errorString, "Error in dscDACalVerify, DSC Error: #%d\n\t%s", result, errorParams.errstring);
		errormessage(errorString);
		return 2;
	}

	error = 0;
	printf("\t");
	printf("D/A Offset Error   D/A Gain Error   Pass/Fail\n");	
	printf("\t");
	printf("       %9.2f        %9.2f        ", (float)dacalparams.offset, (float)dacalparams.gain);
	
	fileprint("\n\tD/A Offset Error   D/A Gain Error   Pass/Fail\n");	
	sprintf(errorString, "\t       %9.2f        %9.2f        ", (float)dacalparams.offset, (float)dacalparams.gain);
	fileprint(errorString);

	if( fabs(dacalparams.gain) > 2 || fabs(dacalparams.offset) > 2)
		error ++;

	if( error > 0)
	{
		printf("FAIL\n");
		fileprint("FAIL\n");
	}
	else
	{
		printf("PASS\n");
		fileprint("PASS\n");
	}

	endScreen();
	return 0;
} // DAAutocal


/*=======================================================
    Dynamically Set TrmiDAC. Allows users to set the
	trimDAC and sample calmux channels to see the effect
========================================================*/

BYTE DynTrimDac(BYTE boardType, DSCB dscb)
{
	ERRPARAMS errorParams;  //structure with error information
	char errorString[500];  //error string
	
	int dac;                //trimDAC to change value on
	int value;              //value to set to trimDAC
	int daoutput;           //D/A output code
	int lowchan, highchan;  //low and high calmux channel to sample
	char key = 'p';
	BYTE result;
	DSCSAMPLE sample;
	DSCSAMPLE sample_values[7];
	DSCADSETTINGS dscadsettings;
	DSCADSCAN dscadscan;
	FLOAT min, max, voltage;
	BYTE admode = 8;
			

	clrscr();

	printf("\n");
	printf("    *================== Dynamic TrimDAC Test ==================*\n\n");
	fileprint("\n    *================== Dynamic TrimDAC Test ==================*\n\n");

	printf("\tThe Dynamic TrimDAC Test allows users to set\n");
	printf("\tDigital to Analog Converter (DAC) to any value\n");
	printf("\tranging from 0-255. Adjusting the TrimDAC values\n");
	printf("\twill affect the A/D and D/A accuracy.\n");

	printf("\n\tWARNING, you are about to change TrimDAC values. This will\n");
	printf("\taffect A/D and D/A accuracy. You may want to exit or\n");
	printf("\tre-calibrate the board after this test.\n\n");


	dscadsettings.addiff = 0;
	dscadsettings.current_channel = 0;
	dscadsettings.gain = 0;
	dscadsettings.load_cal = 1;
	dscadsettings.polarity = 0;
	dscadsettings.range = 1;
	dscadsettings.scan_interval = SCAN_INTERVAL_10;
	
	//Get the A/D mode of DMM48, 8, 9, or 13
	if(boardType == DSC_DMM48)
	{
		admode = getDMM48Mode();
		dscadsettings.gain = admode & 0x3;
		dscadsettings.polarity = (admode >> 2) & 0x1;
		dscadsettings.range = (admode >> 3) & 0x1;
	}


	do{	
		printf("\t");
		printf("Enter 'q' to exit, TrimDAC to be set to continue\n\n");
	
		printf("\t");
		printf("Enter TrimDAC to set (0-7) : ");
		scanf("%d", &dac);
		printf("\t");
		printf("Enter TrimDAC value (0-255) : ");
		scanf("%d", &value);
		
		result = dscSetTrimDac(dscb, dac, value);
		if( result != DE_NONE)
		{
			dscGetLastError(&errorParams);
			sprintf(errorString, "Error in dscSetTrimDac, DSC Error: #%d\n\t%s", result, errorParams.errstring);
			errormessage(errorString);
			return 1;
		}

		printf("\n\tAll values are sampled in A/D +-10V settings\n");
		fileprint("\n\tAll values are sampled in A/D +-10V settings\n");

		//DACs 4-7 control D/A accuracy
		if(dac > 3)
		{
			if( boardType == DSC_HERCEBX)
				dscadsettings.addiff = 1;
			else if( boardType == DSC_DMM32)
			{
				dscadsettings.addiff = 0;
				dscadsettings.current_channel = 7; //calmux channel and is looped to D/A 0
			}
			else
			{
				dscadsettings.addiff = 0;
				dscadsettings.current_channel = 6; //calmux channel and is looped to D/A 0
			}
		}
		else
			dscadsettings.addiff = 0;

		if( dscADSetSettings(dscb, &dscadsettings) != DE_NONE)
		{
			dscGetLastError(&errorParams);
			sprintf(errorString, "Error in dscADSetSettings, DSC Error: #%d\n\t%s", errorParams.ErrCode, errorParams.errstring);
			errormessage(errorString);
			return 2;
		}

		//DACs 4-7 control D/A accuracy
		if( dac > 3)
		{
			DAMinMaxVolt(boardType, &min, &max);

			printf("\n\tD/A Code   A/D Sample   A/D Sample Volt\n");
			fileprint("\n\tD/A Code   A/D Sample   A/D Sample Volt\n");

			//enable calmux
			dscSetCalMux(dscb, 1);

			for ( daoutput=0; daoutput<4096; daoutput+=4095)
			{
				dscDAConvert(dscb, 0, daoutput);

				if( dscADSample(dscb, &sample) != DE_NONE)
				{
					dscGetLastError(&errorParams);
					sprintf(errorString, "Error in dscADSample, DSC Error: #%d\n\t%s", errorParams.ErrCode, errorParams.errstring);
					errormessage(errorString);
					dscSetCalMux(dscb, 0);
					return 3;
				}

				voltage = ADCodeToVolt(boardType, admode, sample);

				printf("\t%8d   %10d   %15.2f\n", daoutput, (int)sample, voltage);
				sprintf(errorString, "\t%8d   %10d   %15.2f\n", daoutput, sample, voltage);
				fileprint(errorString);
			}
		}
		else //DACs 0-3 control A/D accuracy
		{
			//enable calmux
			dscSetCalMux(dscb, 1);

			lowchan = 0;
			if( boardType == DSC_HERCEBX )
				highchan = 3;
			else if( boardType == DSC_DMMAT || boardType == DSC_DMM16AT || boardType == DSC_DMM48 || boardType == DSC_DMM32)
				highchan = 4;

			dscadscan.gain = dscadsettings.gain;
			dscadscan.sample_values = sample_values;
			dscadscan.low_channel = lowchan;
			dscadscan.high_channel = highchan;

			if( dscADScan(dscb, &dscadscan, sample_values) != DE_NONE)
			{
				dscGetLastError(&errorParams);
				sprintf(errorString, "Error in dscADScan, DSC Error: #%d\n\t%s", errorParams.ErrCode, errorParams.errstring);
				errormessage(errorString);
				dscSetCalMux(dscb, 0);
				return 4;
			}
			
			printf("\tMux Channel   A/D Sample   A/D Voltage\n");
			fileprint("\tMux Channel   A/D Sample   A/D Voltage\n");

			for (int numchan=0; numchan<(highchan-lowchan+1); numchan++)
			{
				voltage = ADCodeToVolt(boardType, admode, sample_values[numchan]);

				printf("\t%11d   %10d   %11.2f\n", numchan, sample_values[numchan], voltage);
				sprintf(errorString, "\t%11d   %10d   %11.2f\n", numchan, sample_values[numchan], voltage);
				fileprint(errorString);
			}
		}

		dscSetCalMux(dscb, 0);
		key = getchar() ;
		printf("\n");

	}while(key != 'q');
	
	endScreen();
	return 0;

} //end DynTrimDac


/*=============================================================
	Display TrimDAC Range Effect. Displays the range effect of
	 changing the trimDACs
==============================================================*/

BYTE TrimDACRange(BYTE boardType, DSCB dscb)
{
	ERRPARAMS errorParams;  //structure with error information
	char errorString[500];  //error string

	DSCSAMPLE scan[4];
	DSCSAMPLE scan2[4];
	DSCADSCAN dscadscan;
	DSCADCALPARAMS dscadcalparams;
	DSCADSETTINGS dscadsettings;    //A/D settings structure
	
	clrscr();

	printf("\n");
	printf("    *============= Display TrimDAC Range Affect =============*\n\n");
	fileprint("\n    *============= Display TrimDAC Range Affect =============*\n\n");

	
	for (int mode = 0; mode < 16; mode++)
	{
		//Makes sure only modes that are in effect are tested
		dscadcalparams.adrange = mode;
		if( dscADCalVerify(dscb, &dscadcalparams) == DE_NONE)
		{
			dscadsettings.range = (mode >> 3) & 1;
			dscadsettings.polarity = (mode>>2) & 1 ;
			dscadsettings.gain = mode & 3;
			dscadsettings.load_cal = 0;
			dscadsettings.addiff = 0;
			dscadsettings.scan_interval = SCAN_INTERVAL_9;
			dscadsettings.current_channel = 0;

			if( dscADSetSettings(dscb, &dscadsettings) != DE_NONE)
			{
				dscGetLastError(&errorParams);
				sprintf(errorString, "Error in dscADSetSettings, DSC Error: #%d\n\t%s", errorParams.ErrCode, errorParams.errstring);
				errormessage(errorString);
				dscSetCalMux(dscb, 0);
				return 1;
			}

			printf("\n\tMode   DAC0 DAC1 DAC2 DAC3   CalMux0 CalMux1 CalMux2 CalMux3 \n");
			printf(  "\t-----+-----+----+----+-----+--------+-------+-------+--------\n");

			fileprint("\n\tMode   DAC0 DAC1 DAC2 DAC3   CalMux0 CalMux1 CalMux2 CalMux3 \n");
			fileprint(  "\t-----+-----+----+----+-----+--------+-------+-------+--------\n");

			//setting each individual trimDAC
			for (int dac = 0; dac < 4; dac++)
			{
				//setting all trimDACS to 128.
				SetAllTrimDAC(dscb, 128);
				
				//sets the trimDAC value to 0
				if( dscSetTrimDac(dscb, dac, 0) != DE_NONE)
				{
					dscGetLastError(&errorParams);
					sprintf(errorString, "Error in dscSetTrimDac, DSC Error: #%d\n\t%s", errorParams.ErrCode, errorParams.errstring);
					errormessage(errorString);
					dscSetCalMux(dscb, 0);
					return 2;
				}

				//enable calmux channel
				dscSetCalMux(dscb, 1);

				dscadscan.gain = dscadsettings.gain;
				dscadscan.low_channel = 0;
				dscadscan.high_channel = 3;
				dscadscan.sample_values = scan;
				
				//scan calmux channels
				if( dscADScan(dscb, &dscadscan, scan) != DE_NONE)
				{
					dscGetLastError(&errorParams);
					sprintf(errorString, "Error in dscADScan, DSC Error: #%d\n\t%s", errorParams.ErrCode, errorParams.errstring);
					errormessage(errorString);
					dscSetCalMux(dscb, 0);
					return 3;

				}

				printf("\t %3d    %3d  %3d  %3d  %3d   %7d %7d %7d %7d\n", mode, (dac==0)?0:128, (dac==1)?0:128, (dac==2)?0:128, (dac==3)?0:128, scan[0], scan[1], scan[2], scan[3]);
				sprintf(errorString, "\t %3d    %3d  %3d  %3d  %3d   %7d %7d %7d %7d\n", mode, (dac==0)?0:128, (dac==1)?0:128, (dac==2)?0:128, (dac==3)?0:128, scan[0], scan[1], scan[2], scan[3]);
				fileprint(errorString);

				//sets the trimDAC value to 255
				if( dscSetTrimDac(dscb, dac, 255) != DE_NONE)
				{
					dscGetLastError(&errorParams);
					sprintf(errorString, "Error in dscSetTrimDac, DSC Error: #%d\n\t%s", errorParams.ErrCode, errorParams.errstring);
					errormessage(errorString);
					dscSetCalMux(dscb, 0);
					return 4;
				}

				//enable calmux channel
				dscSetCalMux(dscb, 1);

				//scan calmux channels
				if( dscADScan(dscb, &dscadscan, scan2) != DE_NONE)
				{
					dscGetLastError(&errorParams);
					sprintf(errorString, "Error in dscADScan, DSC Error: #%d\n\t%s", errorParams.ErrCode, errorParams.errstring);
					errormessage(errorString);
					dscSetCalMux(dscb, 0);
					return 5;
				}

				printf("\t %3d    %3d  %3d  %3d  %3d   %7d %7d %7d %7d\n", mode, (dac==0)?255:128, (dac==1)?255:128, (dac==2)?255:128, (dac==3)?255:128, scan2[0], scan2[1], scan2[2], scan2[3]);
				sprintf(errorString, "\t %3d    %3d  %3d  %3d  %3d   %7d %7d %7d %7d\n", mode, (dac==0)?255:128, (dac==1)?255:128, (dac==2)?255:128, (dac==3)?255:128, scan2[0], scan2[1], scan2[2], scan2[3]);
				fileprint(errorString);


				printf("\tDifference :                ");
				fileprint("\tDifference :                ");
				for ( int index=0; index<4; index++)
				{
					printf(" %7d", (scan[index]-scan2[index]));
					sprintf(errorString, " %7d", (scan[index]-scan2[index]));
					fileprint(errorString);
				}

				printf("\n\n");
				fileprint("\n\n");
			}

			if( quitScreen() == 1)
				return 0;

			dscSetCalMux(dscb, 0);
		}
	}

	endScreen();
	return 0;
} //end TrimDACRange
