//////////////////////////////////////////////////////////////////////////////
//	(c) Diamond Systems Corporation 2004
//	
//	http://www.diamondsystems.com
//
//	(c) Copyright 2004 Diamond Systems Corporation. Use of this source code
//	is subject to the terms of Diamond Systems' Software License Agreement.
//	Diamond Systems provides no warranty of proper performance if this
//	source code is modified.
//
//	By KL
//	Updated 1/15/04
///////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include "dscud.h"
#include "datests.h"


/*===============================================
	D/A Test. Performs D/A output and compares
	sampled value to expected values
===============================================*/

BYTE DATest(BYTE boardType, DSCB dscb)
{
	ERRPARAMS errorParams;  //structure with error information
	char errorString[500];  //error string

	
	int i;
	DSCSAMPLE sample;
	long adcode = 0;
	int tolerance = 160;
	long diff;
	BYTE result;
	DSCADSETTINGS dscadsettings;
	DSCDASETTINGS dasettings;
	FLOAT min, max, davolt;
	DSCSAMPLE admax, admin;
	BYTE admode = 8;
	
	clrscr();

	printf("\n");
	printf("    *================== D/A Diagnostic Test ==================*\n\n");

	fileprint("\n    *================== D/A Diagnostic Test ==================*\n\n");


	dscadsettings.range = 1;
	dscadsettings.polarity = 0;
	dscadsettings.gain = 0;
	dscadsettings.load_cal = 1;
	dscadsettings.scan_interval = SCAN_INTERVAL_5;
	dscadsettings.addiff = 1; // this allows for the hercules D/A ch 0 to loop back to any A/D ch
	
	if( boardType == DSC_DMM32)
		dscadsettings.current_channel = 7;
    else if( (boardType == DSC_DMM16AT) || (boardType == DSC_DMMAT) || (boardType == DSC_DMM48))
		dscadsettings.current_channel = 6;
    else
		dscadsettings.current_channel = 0;

	if(boardType == DSC_DMM48)
	{
		//set DMM-48 A/D mode
		admode = getDMM48Mode();
		dscadsettings.gain = (admode & 0x3);
		dscadsettings.polarity = (admode >> 2) & 0x1;
		dscadsettings.range = (admode >> 3) & 0x1;
	}


	if( boardType != DSC_HERCEBX && boardType != DSC_DMM48)
	{
		printf("\tPlease set the D/A setting jumpers to fixed by setting\n\tthe 5 or 10 jumper as shown below.\n");
		printf("\n");
		DAJumperMenu(boardType);
		continueScreen();
	}

	//find D/A range in volts.
	DAMinMaxVolt(boardType, &min, &max);

	if( boardType == DSC_HERCEBX)
	{
		int temp;
		dasettings.load_cal = 1;

		printf("\tEnter (0) for D/A BIPOLAR, (1) for D/A UNIPOLAR : ");
		scanf("%d", &temp);
		getchar();
		printf("\n");

		dasettings.polarity = (BYTE) temp;
		result = dscDASetSettings(dscb, &dasettings);
		if( result != DE_NONE)
		{
			dscGetLastError(&errorParams);
			sprintf(errorString, "Error in dscDASetSettings, DSC Error: #%d\n\t%s\n", result, errorParams.errstring);
			errormessage(errorString);
			return 2;
		}

		if( temp == 0)
			min = -10.0f;
		else min = 0.0f;
		max = 10.0f;
	}

	admax = voltToADCode(boardType, admode, max);
	admin = voltToADCode(boardType, admode, min);
	
	//A/D tolerance versus expected
	tolerance = (int)(((float)admax - (float)admin) / 4096.0f * 8.0f);

	if( ( result = dscADSetSettings( dscb, &dscadsettings ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		sprintf(errorString, "Error in dscADSetSettings, DSC Error: #%d\n\t%s\n", result, errorParams.errstring);
		errormessage(errorString);
		return 1;
	}

	//enable calmux
	dscSetCalMux(dscb, 1);


	printf("\tD/A is detected as %5.2fV to %5.2fV. If not, then board has error or\n\tis not calibrated correctly. Please calibrate board\n\tby running A/D and D/A Autocal Test.\n\n", min, max);

	printf("\t");
	printf("D/A Output   Expected A/D   Sampled A/D   Difference   Pass/Fail\n");
	fileprint("\tD/A Output   Expected A/D   Sampled A/D   Difference   Pass/Fail\n");

	//output 5 different D/A values
	for ( i = 0; i < 5; i ++)
	{
		dscDAConvert(dscb, 0, i * 1000);
		dscSleep(10);
		result = dscADSample(dscb, &sample);
		if( result != DE_NONE)
		{
			dscGetLastError(&errorParams);
			sprintf(errorString, "Error in dscADSample, DSC Error: #%d\n\t%s\n", result, errorParams.errstring);
			errormessage(errorString);
			dscSetCalMux(dscb, 0);
			return 3;
		}

		//calculate the D/A voltage that's being output
		davolt = ((float)(i*1000.0f)/4096.0f) * (float)fabs(max - min);
		
		//if D/A is in bipolar mode, do the following
		if( min < 0)
			davolt -= fabs(min);

		adcode = voltToADCode(boardType, admode, davolt);
		
		diff = labs(adcode - (long)sample);

		printf("\t");
		printf("     %5d         %6d        %6d       %6ld        ", i*1000, (int)adcode, sample, diff);
		sprintf(errorString, "\t     %5d         %6d        %6d       %6ld        ", i*1000, (int)adcode, sample, diff);
		fileprint(errorString);

		if( diff > tolerance)
		{
			printf("FAIL\n");
			fileprint("FAIL\n");
		}
		else
		{
			printf("PASS\n");
			fileprint("PASS\n");
		}
	}
	
	dscSetCalMux(dscb, 0);
	endScreen();
	return 0;
}//end DATest


/*==================================================
	D/A User Test. Allows user to output D/A code on 
	any D/A channel
===================================================*/

BYTE DAUserTest(BYTE boardType, DSCB dscb)
{
	ERRPARAMS errorParams;  //structure with error information
	char errorString[500];  //error string
	
	int dacode;          //D/A code to output
	int channel;         //D/A channel to output
	BYTE result;
	float voltage;
	int temp;
	FLOAT min;           //minimum D/A output voltage
	FLOAT max;           //maximum D/A output voltage
	DSCDASETTINGS dasettings;

	//clear screen
	clrscr();

	printf("\n");
	printf("    *==================== D/A User Test ====================*\n\n");
	fileprint("\n    *==================== D/A User Test ====================*\n\n");

	
	printf("\tIf test fails, it might be because of calibration error.\n\tPlease calibrate board by running A/D and D/A Autocal Test.\n\n");

	do
	{
		//set HERC_EBX D/A polarity
		if( boardType == DSC_HERCEBX)
		{
			printf("\n\tEnter (0) for Bipolar, (1) for Unipolar : ");
			scanf("%d", &temp);
			getchar();
			dasettings.load_cal = 1;
			dasettings.polarity = (BYTE) temp;
			if( (result = dscDASetSettings(dscb, &dasettings)) != DE_NONE)
			{
				dscGetLastError(&errorParams);
				sprintf(errorString, "dscDASetSettings Error, DSC Error: #%d\n\t#%s\n", result, errorParams.errstring);
				errormessage(errorString);
				return 1;
			}
		}
	
		printf("\n\tEnter D/A channel to output or 4 to exit\n\n");
		printf("\t");
		printf("Enter D/A channel : ");
		scanf("%d", &channel);
		if( channel == 4)
			break;
		printf("\t");
		printf("Enter D/A code : ");
		scanf("%d", &dacode);
		getchar();

		result = DAMinMaxVolt(boardType, &min, &max);

		if( result != DE_NONE && result != 1)
			return 200;
		
		result = dscDAConvert(dscb, channel, dacode);
		if( result != DE_NONE)
		{
			dscGetLastError(&errorParams);
			sprintf(errorString, "Error in dscDAConvert, DSC Error: #%d\n\t%s\n", result, errorParams.errstring);
			errormessage(errorString);
			return 1;
		}
	
		//compute the D/A voltage
		if( boardType == DSC_HERCEBX)
		{
			if( dasettings.polarity == 1)
			{
				voltage = ((FLOAT)dacode - 2048.0f) / 2048.0f * 10.0f;
				printf("\t");
				printf("Expected A/D voltage for +-10V D/A output: %6.3fV\n", voltage);
				sprintf(errorString, "\tExpected A/D voltage for +-10 D/A output: %6.3f\n", voltage);
				fileprint(errorString);
			}
			else
			{
				voltage = dacode / 4096.0f * 10.0f;
				printf("\t");
				printf("Expected A/D voltage for 0V to 10V D/A output: %6.3fV\n", voltage);
				sprintf(errorString, "\tExpected A/D voltage for 0-10 D/A output: %6.3f\n", voltage);
				fileprint(errorString);
			}
		}
		else
		{
			voltage = dacode / 4096.0 * fabs(max - min);
			
			if( min < 0)
				voltage -= fabs(min);

			printf("\t");
			printf("Expected A/D voltage for %5.2fV to %4.2fV D/A output: %6.3fV\n", min, max, voltage);
			sprintf(errorString, "\tExpected A/D voltage for +-5V D/A output: %6.3f\n", voltage);
			fileprint(errorString);
		}
		
		
		dscSleep(10);
	} while(1);

	return 0;
}
