//////////////////////////////////////////////////////////////////////////////
//	(c) Diamond Systems Corporation 2004
//	
//	http://www.diamondsystems.com
//
//	(c) Copyright 2004 Diamond Systems Corporation. Use of this source code
//	is subject to the terms of Diamond Systems' Software License Agreement.
//	Diamond Systems provides no warranty of proper performance if this
//	source code is modified.
//
//	By KL
//	Updated 1/15/04
///////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include <dos.h>
#include "dscud.h"
#include "adtests.h"
#include "datests.h"
#include "interrupt.h"
#include "autocal.h"
#include "diagnostic.h"


ERRPARAMS errorParams; //variable that spits out errors
DSCB dscb;             //board reference 
DSCCB dsccb;           //structure that contains information for board initialization
BYTE DMM48Mode;        //DMM-48-AT A/D Mode, 8, 9, or 13
FLOAT refs[5];         //array for storing reference voltage values
char errorString[500]; //error string for display
char errorPrint[500];  //error string for printing
FILE * logfile;        //file pointer handler
bool logBool;          //boolean variable determining if file for printing exists

//MAIN()
int main()
{
	int selection;
	BYTE result;
	BYTE boardtype;

	logBool = false;

	if( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "Error initializing driver %s\n", errorParams.errstring );
		endScreen();
		return 0;
	}

	while(1)
	{
		selection = menu();

		if( selection == 0)
		{
			dscFree();

			if( logBool)
				fclose(logfile);
			return 0;
		}

		//initialize board
		switch(selection)
		{
		case 1:
			result = InitBoard(DSC_DMMAT);
			boardtype = DSC_DMMAT;
			fileprint("\tDMM-AT Test\n");
			break;
		case 2:
			result = InitBoard(DSC_DMM16AT);
			boardtype = DSC_DMM16AT;
			fileprint("\tDMM-16-AT Test\n");
			break;
		case 3:
			result = InitBoard(DSC_DMM32);
			boardtype = DSC_DMM32;
			fileprint("\tDMM-32 Test\n");
			break;
		case 4:
      		result = InitBoard(DSC_DMM48);
			boardtype = DSC_DMM48;
			fileprint("\tDMM-48 Test\n");
			break;
		case 5:
			result = InitBoard(DSC_HERCEBX);
			boardtype = DSC_HERCEBX;
			fileprint("\tHERCEBX Test\n");
			break;
		default:
			printf("\tWRONG OPTION, no option %d\n", (int)selection);
			result = 255;
			break;
		}

		if( result == 255)
		{
			endScreen();
			continue;
		}

		if( result != DE_NONE)
		{
			dscGetLastError(&errorParams);
			printf("\n\t");
			printf("Error in dscInitBoard, DSC Error: #%d\n\t%s\n",  result, errorParams.errstring);
			if( logBool)
				fprintf(logfile, "\n\tError in dscInitBoard, DSC Error: #%d\n\t%s\n", result, errorParams.errstring);
			endScreen();
			continue;
		}

		referenceVoltages(0); //get reference voltage
		if(boardtype == DSC_DMM48)
			getDMM48Mode(); //get DMM48 A/D Mode
		DiagTest(boardtype); //start rest of tests
	}
}

/*=============================================
	Board Menu
===============================================*/

int menu()
{
	int result;
	
	clrscr();
	//printf("\n*========================================================*\n\n"); //60 characters including the 2 '*'
	printf("\n");
	printf("  *================== Diamond Systems Diagnostic Test ==================*\n"); // 73 characters
	printf("              =========== version 2, 1/1/04, by KL ===========\n\n");
	printf("\t"); printf("Please choose one of the following options\n\n");
	printf("\t"); printf("0. Exit\n");
	printf("\t"); printf("1. DMM-AT Test\n");
	printf("\t"); printf("2. DMM-16-AT Test\n");
	printf("\t"); printf("3. DMM-32-AT Test\n");
	printf("\t"); printf("4. DMM-48-AT Test\n");
	printf("\t"); printf("5. HERCULES Test\n");

	printf("\n");
	printf("\t");
	scanf("%d", &result);
	getchar();
	
	return result;
}


/*=============================================
	Initialize the board
==============================================*/

BYTE InitBoard(BYTE boardType)
{
	char filename[50];
	char input[100];
	int numchar;

	int tt;

	printf("\n");
	dsccb.boardtype = boardType;
	dsccb.base_address = 0;
	dsccb.int_level = 0;

	printf("\t");

	if( boardType == DSC_HERCEBX)
	{
		memset(input, 0, sizeof(char) * 100);
		printf("Enter the base address in hex (default 240) : ");
		numchar = optionInput(input);

		if( numchar == 0)
			dsccb.base_address = 0x240;	
		else
		{
			sscanf(input, "%x", &tt);
			dsccb.base_address = tt;	
		}
		
		memset(input, 0, sizeof(char) * 100);
		printf("\t"); printf("Enter the interrupt level (default 5) : ");
		numchar = optionInput(input);

		if( numchar == 0)
			dsccb.int_level = 5;	
		else
		{
      		sscanf(input, "%d", &tt);
			dsccb.int_level = tt;	
		}
	}
	else
	{
		memset(input, 0, sizeof(char) * 100);
		printf("Enter the base address in hex (default 300) : ");
		numchar = optionInput(input);

		if( numchar == 0)
			dsccb.base_address = 0x300;	
		else
		{
			sscanf(input, "%x", &tt);
			dsccb.base_address = tt;	
		}
		
		memset(input, 0, sizeof(char) * 100);
		printf("\t"); printf("Enter the interrupt level (default 7) : ");
		numchar = optionInput(input);

		if( numchar == 0)
			dsccb.int_level = 7;
		else
		{
			sscanf(input, "%d", &tt);
			dsccb.int_level = tt;	
		}
	}
	
	dsccb.dma_level = 3;

	dsccb.clock_freq = 10000000L;

	logBool = true;
	sprintf(filename, "Diamond_Diagnostic.txt");

	logfile = fopen(filename, "a+");

	if( logfile == NULL)
	{
		logBool = false;
		printf("\n\t"); printf("Can't open file\n");
		continueScreen();
	}
	else {
		time_t timer;
		time(&timer);
		sprintf(errorString, "\n\n    ****** Diamond Systems Diagnostic Test ******\n       %s\n", ctime(&timer));
		fileprint(errorString);
	}
	
	return dscInitBoard(boardType, &dsccb, &dscb);
} //end initboard



/*=============================================
	Starts the rest of the tests
=============================================*/

BYTE DiagTest(BYTE boardType)
{
	BYTE result;
	int selection;

	enum menuOptions {  exit,
						eprom,
						refvoltage,
						adautocal, 
						daautocal, 
						dyntrimdac,
						trimdacrange,
						adtest,
						adtestuser,
						datest,
						datestuser,
						diotestuser,
						inttest,
						inttestuser,
						timertest 
						} ;

	do{
		selection = DiagMenu(boardType);

		if( logBool)
			fflush(logfile);

		switch(selection)
		{
		case exit:
			return 0;
		case eprom:
			result = eeprom(boardType);
			if ( result != 0)
			{
				printf("\n\t%s\n", errorString);
				sprintf(errorPrint, "\n\t%s\n", errorString);
				fileprint(errorPrint);

				endScreen();
			}
			break;
		case refvoltage:
			result = refVoltage(boardType);
			if( result != 0)
			{
				printf("\n\t%s\n", errorString);
				sprintf(errorPrint, "\n\t%s\n", errorString);
				fileprint(errorPrint);
				
				endScreen();							
			}
			break;
		case adautocal:
			result = ADAutocal(boardType, dscb);
			if( result != 0)
			{
				printf("\n\t%s\n", errorString);
				sprintf(errorPrint, "\n\t%s\n", errorString);
				fileprint(errorPrint);

				endScreen();
			}
			break;
		case daautocal:
			result = DAAutocal(boardType, dscb);
			if( result != 0)
			{
				printf("\n\t%s\n", errorString);
				sprintf(errorPrint, "\n\t%s\n", errorString);
				fileprint(errorPrint);

				endScreen();
			}
			break;
		case dyntrimdac :
			result = DynTrimDac(boardType, dscb);
			if( result != 0)
			{
				printf("\n\t%s\n", errorString);
				sprintf(errorPrint, "\n\t%s\n", errorString);
				fileprint(errorPrint);

				endScreen();
			}
			break;
		case trimdacrange:
				result = TrimDACRange(boardType, dscb);
				if( result != DE_NONE)
				{
					printf("\n\t%s\n", errorString);
					sprintf(errorPrint, "\n\t%s\n", errorString);
					fileprint(errorPrint);
					endScreen();
				}
			break;
		case adtest:
			result = ADTest(boardType, dscb);
			if ( result != 0)
			{
				printf("\n\t%s\n", errorString);
				sprintf(errorPrint, "\n\t%s\n", errorString);
				fileprint(errorPrint);

				endScreen();
			}
			break;
		case adtestuser:
			result = ADUserTest(boardType, dscb);
			if ( result != 0)
			{
				printf("\n\t%s\n", errorString);
				sprintf(errorPrint, "\n\t%s\n", errorString);
				fileprint(errorPrint);

				endScreen();
			}
			break;
		case datest:
			result = DATest(boardType, dscb);
			if ( result != 0)
			{
				printf("\n\t%s\n", errorString);
				sprintf(errorPrint, "\n\t%s\n", errorString);
				fileprint(errorPrint);

				endScreen();
			}
			break;
		case datestuser:
			result = DAUserTest(boardType, dscb);
			if ( result != 0)
			{
				printf("\n\t%s\n", errorString);
				sprintf(errorPrint, "\n\t%s\n", errorString);
				fileprint(errorPrint);

				endScreen();
			}
			break;
		case diotestuser:
			result = DIOUserTest(boardType, dscb);
			if ( result != 0)
			{
				printf("\n\t%s\n", errorString);
				sprintf(errorPrint, "\n\t%s\n", errorString);
				fileprint(errorPrint);

				endScreen();
			}
			break;
		case inttest:
			result = IntTest(boardType, dscb);
			if ( result != 0)
			{
				printf("\n\t%s\n", errorString);
				sprintf(errorPrint, "\n\t%s\n", errorString);
				fileprint(errorPrint);

				endScreen();
			}
			break;
		case inttestuser:
			result = IntUserTest(boardType, dscb);
			if ( result != 0)
			{
				printf("\n\t%s\n", errorString);
				sprintf(errorPrint, "\n\t%s\n", errorString);
				fileprint(errorPrint);

				endScreen();
			}
			break;
		case timertest:
			result = TimerTest(boardType, dscb);
			if ( result != 0)
			{
				printf("\n\t%s\n", errorString);
				sprintf(errorPrint, "\n\t%s\n", errorString);
				fileprint(errorPrint);

				endScreen();
			}
			break;
		default:
			printf("\tWRONG OPTION, no option %d\n", selection );
			endScreen();
		}
	}  while(1);
}



/*=============================================
	Diagnostic Test Menu
=============================================*/

BYTE DiagMenu(BYTE boardtype)
{
	int result;

	clrscr();

	printf("\n");
	printf("   *======================= ");

	switch(boardtype)
	{
	case DSC_DMMAT:
		printf("DMM-AT Test");
		break;
	case DSC_DMM16AT:
		printf("DMM-16-AT Test");
		break;
	case DSC_DMM32:
		printf("DMM-32-AT Test");
		break;
	case DSC_DMM48:
		printf("DMM-48-AT Test");
		break;
	case DSC_HERCEBX:
		printf("HERCULES Test");
		break;
	default:
		printf("WRONG OPTION, no option %d\n", (int)boardtype);
		printf(" =======================*\n\n");
		endScreen();
		return 255;
	}

	printf(" =======================*\n\n");
	
	printf("\t"); printf("Please choose one of the following options\n\n");
	printf("\t"); printf("0.  Exit\n");						    // exit
	printf("\t"); printf("1.  Display EEPROM and Reference Voltages\n");	// eeprom
	printf("\t"); printf("2.  Set Reference Voltage\n");	    // refvoltage
	printf("\t"); printf("3.  A/D Autocal Test\n");				// AD autocal
	printf("\t"); printf("4.  D/A Autocal Test\n");				// D/A autocal
	printf("\t"); printf("5.  Dynamically Set TrimDAC\n");		// Dynamic Trimdac setting
	printf("\t"); printf("6.  Display TrimDAC Range Effect\n");	// Displays Trimdac range affect
	printf("\t"); printf("7.  A/D 'QUICK' Diagnostic Test\n");	// adtest
	printf("\t"); printf("8.  A/D User Select Test\n");		    // adtestuser
	printf("\t"); printf("9.  D/A 'QUICK' Diagnostic Test\n");	// datest
	printf("\t"); printf("10. D/A User Select Test\n");		    // datestuser
	printf("\t"); printf("11. Digital I/O User Test\n");		// diousertest
	printf("\t"); printf("12. A/D 'QUICK' Interrupt Diagnostic Test\n");	// inttest
	printf("\t"); printf("13. A/D Interrupt User Select Test\n");	// inttestuser
	printf("\t"); printf("14. Timer Interrupt Diagnostic Test\n");  // timertest

	printf("\n\t"); scanf("%d", &result);
	getchar();

	return (BYTE) result;
}


/*===============================================
	DIO User Test. Allows users to test 
	Digital input, output, and pullup/pulldown
=================================================*/

BYTE DIOUserTest(BYTE boardType, DSCB dscb)
{
  	int diocodeout;
	BYTE diocode;
	int port;
	int option;
	BYTE result;
	BYTE *config;

	clrscr();
	printf("\n    *=================Digital I/O User Test=================*\n\n");
	fileprint("\n    *=================Digital I/O User Test=================*\n\n");

	//config byte
	config = (BYTE*)malloc(sizeof(BYTE));

	//option 0 for output, 1 for input, 2 for pullup/down
   	if( boardType == DSC_DMM32 || boardType == DSC_HERCEBX)
	{
		printf("\t"); printf("Enter (0) for output, (1) for input, (2) for pull up/down mode : ");
		scanf("%d", &option);
		getchar();
    }
    else
    {
     	printf("\t"); printf("Enter (0) for output, (1) for input : ");
		scanf("%d", &option);
		getchar();
	}

	//boards that need to set the digital input/output configuration
	if(boardType == DSC_DMM32 || boardType == DSC_HERCEBX || boardType == DSC_DMM48)
	{
		if(boardType == DSC_DMM32)
		{
			if(option == 0)
				*config = 0x80;
			else
				*config = 0x9B;
		}
		else if(boardType == DSC_HERCEBX)
		{
			if(option == 0)
				*config = 0x1F;
			else
				*config = 0x00;
		}
		else if(boardType == DSC_DMM48)
		{
			if(option == 0)
				*config = 0xf;
			else
				*config = 0x0;
		}

		result = dscDIOSetConfig(dscb, config);
		if( result != DE_NONE)
		{
			dscGetLastError(&errorParams);
			sprintf(errorString, "Error in dscDIOSetConfig, DSC Error: #%d\n\t%s\n", result, errorParams.errstring);
			free(config);
			return 1;
		}
	}

	if( option == 0) //running option 0, output
	{
		printf("\n\tPlease use a voltmeter to measure the digitial output pins\n");

		do{
			printf("\n");
			if( boardType == DSC_DMM32 || boardType == DSC_HERCEBX)
			{
				int maxport = 2;

				if( boardType == DSC_HERCEBX)
					maxport = 4;
				printf("\t"); printf("Enter Digital I/O port to output on (0-%d) : ", maxport);
				scanf("%d", &port);
				getchar();
			}
			else
				port = 0;
		
			printf("\t"); printf("Enter Digital I/O code to output (0-255) : ");
			scanf("%d", &diocodeout);
			getchar();
			result = dscDIOOutputByte(dscb, port, (BYTE)diocodeout);
		}while(!quitScreen());
		return 0;
	}
	else if( option == 1)  //running option 1, input
	{
		printf("\n\tPlease input voltage to the digital input pins\n");

		unsigned int loop = 0;
		printf("\n\t"); printf("Press ENTER to stop\n\n");
		printf("\t");
		printf("Port  Input Code  Bit7  Bit6  Bit5  Bit4  Bit3  Bit2  Bit1  Bit0\n\n");
		fileprint("\n\tPort  Input Code  Bit7  Bit6  Bit5  Bit4  Bit3  Bit2  Bit1  Bit0\n\n");

		do{
			if( loop > 6)
			{
				loop = 0;
				printf("\n\t"); printf("Press ENTER to stop\n\n");
				printf("\t");
				printf("Port  Input Code  Bit7  Bit6  Bit5  Bit4  Bit3  Bit2  Bit1  Bit0\n\n");
				fileprint("\n\tPort  Input Code  Bit7  Bit6  Bit5  Bit4  Bit3  Bit2  Bit1  Bit0\n\n");
			}

			int totalPorts = 1;
			if( boardType == DSC_DMM32)
				totalPorts = 3;
			else if( boardType == DSC_HERCEBX)
				totalPorts = 5;
			else
				totalPorts = 1;

			for ( int i=0; i < totalPorts; i++)
			{
				char charPort = '0';

				if( (boardType == DSC_DMM32) || (boardType == DSC_HERCEBX))
				{
					if( i == 0 )
						charPort = 'A';
					else if( i == 1)
						charPort = 'B';
					else if( i == 2)
						charPort = 'C';
					else if( i == 3)
						charPort = 'D';
					else if( i == 4)
						charPort = 'E';
				}

				result = dscDIOInputByte(dscb, i, &diocode);
				printf("\t");
				printf("  %c     %3d       %3d   %3d   %3d   %3d   %3d   %3d   %3d   %3d\n",
								charPort, diocode, (diocode&0x80) >> 7, (diocode & 0x40) >> 6, (diocode & 0x20) >> 5,
								(diocode & 0x10) >> 4, (diocode & 0x08) >> 3, (diocode & 0x04) >> 2, (diocode & 0x02) >> 1, (diocode & 0x01) >> 0);

				sprintf(errorString, "\t  %c     %3d        %3d   %3d   %3d   %3d   %3d   %3d   %3d   %3d\n",
								charPort, diocode, (diocode&0x80) >> 7, (diocode & 0x40) >> 6, (diocode & 0x20) >> 5,
								(diocode & 0x10) >> 4, (diocode & 0x08) >> 3, (diocode & 0x04) >> 2, (diocode & 0x02) >> 1, (diocode & 0x01) >> 0);
				fileprint(errorString);
			}

			loop += totalPorts;

			printf("\n");
			dscSleep(totalPorts * 750);

		} while(!kbhit());

		getchar();
	}
	else if( option == 2 && (boardType == DSC_DMM32 || boardType == DSC_HERCEBX)) //running option 2, pull up/down
	{
		printf("\n\t"); printf("Please remove everything attached to Digital I/O ports \n\tthen press ENTER\n");
		getchar();

		int value = 0;
		int maxPorts;

		if( boardType == DSC_DMM32)
			maxPorts = 3;
		else if( boardType == DSC_HERCEBX)
			maxPorts = 5;	

		for ( int i=0; i < maxPorts; i++)
		{
			dscDIOInputByte(dscb, i, &diocode);

			if(diocode == 255)
				value++;
			else if(diocode != 0)
				value += 7;
		}

		if(value == 3 || value == 5)
		{
			printf("\tDigital I/O is in PULL UP MODE\n");
			fileprint("\tDigital I/O is in PULL UP MODE\n");
		}                                  
		else if( value == 0)
		{
			printf("\tDigital I/O is in PULL DOWN MODE\n");
			fileprint("\tDigital I/O is in PULL DOWN MODE\n");
		}
		else
		{
			printf("\tDigital I/O error in pull down/up test,\n\tsome ports pull up, some pull down\n");
			fileprint("\tDIO error in pull down/up test,\n\tsome ports pull up, some pull down\n");
		}
	}

	endScreen();
	return 0;
}

/*////////////////////////////////////////////////////////////////////////////
//
//	Helper Functions
//
////////////////////////////////////////////////////////////////////////////*/


/*============================================
	Converts voltage to A/D Code
==============================================*/

DSCSAMPLE voltToADCode(BYTE boardType, BYTE mode, FLOAT volt)
{
	BYTE range, polarity, gain;
	FLOAT adcode;

	gain = (mode&0x3);
	polarity = (mode>>2)&0x1;
	range = (mode>>3)&0x1;

	if(boardType == DSC_DMMAT) //DMMAT A/D code range is 0-4095, different from other boards
	{
		if(polarity == 1)
			adcode = volt/((range==1)?10.0f:5.0f) * 4096.0f;
		else
			adcode = (volt/((range==1)?10.0f:5.0f) * 2048.0f) + 2048.0f;
	}
	else if( (boardType == DSC_DMM16AT) || (boardType == DSC_DMM32) || (boardType == DSC_HERCEBX) || (boardType == DSC_DMM48))
	{ //A/D code range from -32768 to 32767.

		if(polarity == 1)
			adcode = volt/((range==1)?10.0f:5.0f) * 65536.0f;
		else
			adcode = volt/((range==1)?10.0f:5.0f) * 32768.0f;
	}

	adcode *= (FLOAT)pow(2,gain);

	if(boardType == DSC_DMMAT && adcode > 4095.0f)
		adcode = 4095.0f;
	else if(adcode > 32767.0f)
		adcode = 32767.0f;

	return (DSCSAMPLE) adcode;

} // volttoadcode


/*==================================================
	Converts A/D code to voltage
==================================================*/

FLOAT ADCodeToVolt(BYTE boardType, BYTE mode, DSCSAMPLE adcode)
{
	BYTE range, polarity, gain;
	FLOAT volt;

	gain = (mode&0x3);
	polarity = (mode>>2)&0x1;
	range = (mode>>3)&0x1;

	if(boardType == DSC_DMMAT) //DMMAT A/D code range is 0-4095, different from other boards
	{
		if(polarity == 1)
			volt = ((FLOAT)adcode)/4096.0f;
		else
			volt = ((FLOAT)adcode-2048.0f)/2048.0f;
	}
	else //Other boards A/D range is -32768 to 32767
	{
		if(polarity == 1)
			volt = ((FLOAT)adcode+32768.0f)/65536.0f;
		else
			volt = ((FLOAT)adcode)/32768.0f;
	}

	volt *= ((range==1)?10.0f:5.0f);
	volt /= (FLOAT)pow(2, gain);
	return volt;
}


/*=============================================================
	EEPROM Test. Displays EEPROM content and reference voltages
===============================================================*/

BYTE eeprom(BYTE boardType)
{
	clrscr();
	printf("\n");
	printf("   *============== Display EEPROM and Reference Voltages ==============*\n");
	fileprint("\n   *============== Display EEPROM and Reference Voltages ==============*\n");
	displayEEPROMInfo(1); //displays EEPROM content
	continueScreen();
	referenceVoltages(1); //displays reference voltages
	endScreen();
	return DE_NONE;
}


/*===================================================================
	Set Reference Voltage Test. Displays and sets reference voltages
====================================================================*/

BYTE refVoltage(BYTE boardType)
{
	BYTE result;

	clrscr();

	printf("\n");
	printf("   *============== Set Reference Voltage ==============*\n\n");
	fileprint("\n   *============== Set Reference Voltage ==============*\n\n");
	printf("\t");

	referenceVoltages(1); //display reference voltages

	if( quitScreen() == 1)
		return 0;

	printf("\n");

	result = measureReference(boardType); //users measure and set reference voltages
	endScreen();
	return result;
}


/*===================================================================
	Allows users to measure and set reference voltages
====================================================================*/

BYTE measureReference(BYTE boardType)
{
	BYTE result;
	DFLOAT tempRef[7];
	int i;
	float tempf;
	int maxRefs;

	maxRefs = 5;

	printf("\t"); 

	if( boardType == DSC_DMM32)
	{
		printf("Locate J9 on the DMM32. Place the ground probe of your voltmeter\n" );
		printf("\t"); 
		printf("on the negative pin and the other probe on the positive pin\n");
	}
	else if( boardType == DSC_DMM16AT)
	{
		printf( "Locate J7 on the DMM16AT. Place the ground probe of your voltmeter\n" );
		printf("\t"); printf( "on the G pin and the other probe on the M pin\n" );
	}
	else if( boardType == DSC_DMMAT)
	{
		printf( "Locate J7 on the DMMAT. Place the ground probe of your voltmeter \n" );
		printf("\t"); printf( "on the G pin and the other probe on the M pin\n" );
	}
	else if( boardType == DSC_HERCEBX)
	{
		printf("Locate the T2 hearder. Place the probes in their correpsonding spots\n");
		maxRefs = 4;
	}

	continueScreen();

	//enable calmux
	dscSetCalMux(dscb, 1); 

	for ( i = 0; i < 7; i++)
		tempRef[i] = 0;

	//asking user to measure and set reference voltages
	for( i = 0; i < maxRefs; i++ )
	{
		if( (result = dscADSetChannel(dscb, (unsigned char) i, (unsigned char)i)) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			sprintf(errorString, "Error in dscADSetChannel, DSC Error: #%d\n\t%s\n", result, errorParams.errstring);
			dscSetCalMux(dscb, 0);
			return 1;
		}
		
		printf("\t");
		printf( "Enter the voltage on the voltmeter (cal mux chan %d) : ", i );
		scanf( "%f", &tempf);
		getchar();
		refs[i] = (FLOAT)tempf;
		tempRef[i] = tempf;
	}

	//disable calmux
	dscSetCalMux(dscb, 0);

	//setting reference voltages
	if( dscSetReferenceVoltages( dscb, tempRef) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		sprintf(errorString, "Error in dscSetReferenceVoltages, DSC Error: #%d\n\t%s\n", result, errorParams.errstring);
		return 2;
	}

	for( i = 0; i < maxRefs; i++)
		tempRef[i] =0;

	//getting reference voltages for display. This checks if the reference
	//voltages previously set were stored correctly
	if( dscGetReferenceVoltages( dscb, tempRef) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		sprintf(errorString, "Error in dscGetreferenceVoltages, DSC Errr: #%d\n\t%s\n", result, errorParams.errstring);
		return 3;
	}

	for( i = 0; i < maxRefs; i++)
	{
		printf("\t");
		printf("New Reference #%d, %8.4fV\n", i, (float)tempRef[i]);
		sprintf(errorString, "\tNew Reference #%d, %8.4fV\n", i, (float)tempRef[i]);
		fileprint(errorString);
	}
	
	return 0;
}


/*===================================================================
	Asks user to press enter to continue
====================================================================*/

void continueScreen()
{
	while( kbhit())
		getchar();
	printf("\n\tPress ENTER to continue\n");
	getchar();

	while(kbhit())
		getchar();
}


/*===================================================================
	Asks user to press enter to end and return to previous menu
====================================================================*/

void endScreen()
{
	while( kbhit())
		getchar();
	printf("\n\tPress ENTER to return to menu\n");
	getchar();
	while(kbhit())
		getchar();
}


/*===================================================================
	Asks user to press enter to exit or space to continue
====================================================================*/

int quitScreen()
{
	char quit ;

	while( kbhit())
		getchar();
	printf("\n\tPress SPACE BAR to continue or ENTER to return to menu\n");
	
	while(!kbhit());
	
	quit = getch();

	if( ((int) quit) != 32)  // space bar is ASCII 32
		return 1;
	else return 0;
}


/*===================================================================
	Prints out D/A jumper configurations for each board
====================================================================*/

void DAJumperMenu(BYTE boardType)
{
	if( boardType == DSC_DMM32)
	{
		printf("\tFor jumper J6\n");
		printf("\t10    out\n");
		printf("\t5     in\n");
		printf("\tP     out\n");
		printf("\tB     in\n");
		printf("\tR     out\n");
	}
	else if( boardType == DSC_DMM16AT)
	{
		printf("\tFor jumper J5\n");
		printf("\t5     in\n");
		printf("\tP     out\n");
		printf("\tB     in\n");
		printf("\tU     out\n");
	}
	else if( boardType == DSC_DMMAT)
	{
		printf("\tFor jumper J5\n");
		printf("\t5     in\n");
		printf("\tP     out\n");
		printf("\tN     in\n");
		printf("\tX     out\n");
		printf("\tB     in\n");
	}
}


/*===================================================================
	Display EEPROM content
	Param: d. If d is true, print out EEPROM content
====================================================================*/

#define REF_BASE 32

void displayEEPROMInfo(int d)
{
	BYTE value;

	printf("\n\t");
	if(d)
	{
		printf("EEPROM Contents in HEX Value :\n\n");
		fileprint("\n\tEEPROM Contents in HEX Value :\n\n");
	}
	else 
	{
		printf("EEPROM Contents in HEX Value :\n\n");
		fileprint("\n\tEEPROM Contents in HEX Value :\n\n");
	}

	//goes through and displays EEPROM content from address 0-128
	for (int x = 0; x < 128; x++)
	{
		dscGetEEPROM(dscb, x, &value);

		if(x % 8 == 0){
			printf("\t");
			printf("Addr. 0x%02x - 0x%02x:  ",x, x + 8);
			if( logBool)
				fprintf(logfile, "\tAddr. 0x%04x - 0x%04x:  ",x, x + 8);
		}
		if(d) 
		{
			printf("%02X ",value);
			if( logBool)
				fprintf(logfile, "%02X ",value);
		}
		else 
		{
			printf("%03d ",value);
				if( logBool)
					fprintf(logfile, "%03d ",value);
		}

		if((x+1) % 4 == 0 && !(x % 8 == 0)) {
			printf("   ");
			fileprint("   ");
		}

		if((x+1) % 8 == 0) {
			printf("\n");
			fileprint("\n");
		}
	}

	printf("\n");
	fileprint("\n");
}


/*=======================================================================
	Display reference voltages
	Param: Display. If display is true, then print out reference voltages
=========================================================================*/

BYTE referenceVoltages(int display)
{
	if( display)
		printf("\n\tCurrent Reference Voltages :\n\n");

	BYTE maxRefs;
	DFLOAT references[7];

	if( dsccb.boardtype == DSC_HERCEBX)
		maxRefs = 4;
	else maxRefs = 5;

	if( display)
	{
		printf("\tReference #   Stored Voltage\n");
		fileprint("\tReference #   Stored Voltage\n");
	}

	if(dscGetReferenceVoltages(dscb, references) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		sprintf(errorString, "Error in dscGetReferencVoltage, DSC Error: #%d\n\t%s\n", errorParams.ErrCode, errorParams.errstring);
		errormessage(errorString);
		printf(errorString);
		return 1;
	}

	for (int x = 0; x < maxRefs; x++)
	{
		refs[x] = (FLOAT)references[x];

 		if( display)
		{
			printf("\t    %4d         %7.3f V\n",x,refs[x]);
			sprintf(errorString, "\t    %4d         %7.3f V\n",x,refs[x]);
			fileprint(errorString);
		}
	}

	if( display)
	{
		printf("\n");
		fileprint("\n");
	}

	return 0;
}


/*=================================================================================
	Finds and returns the D/A output range.
	Param: min. The minimum D/A output voltage.
	Param: max. The maximum D/A output voltage.
	Example: If D/A output range is +-5V, the min would return -5V, max +5V.
===================================================================================*/

BYTE DAMinMaxVolt(BYTE boardType, FLOAT * min, FLOAT * max)
{
	int muxchan = 0;
	DSCADSETTINGS dscadsettings;
	DSCSAMPLE adsampMin;
	DSCSAMPLE adsampMax;
	BYTE admode = 8;

	if( boardType == DSC_DMM32)
		muxchan = 7;
	else if( boardType == DSC_HERCEBX)
	{
		muxchan = 0;
		return 1;
	}
	else 
		muxchan = 6;

	dscadsettings.addiff = 0;
	dscadsettings.current_channel = muxchan;
	dscadsettings.gain = 0;
	dscadsettings.load_cal = 1;
	dscadsettings.polarity = 0;
	dscadsettings.range = 1;
	dscadsettings.scan_interval = SCAN_INTERVAL_5;

	if(boardType == DSC_DMM48)
	{
		//DMM-48-AT has only 3 modes, 8, 9, 13 and some of boards only support one mode
		admode = getDMM48Mode();
		dscadsettings.gain = (admode & 0x3);
		dscadsettings.polarity = (admode >> 2) & 0x1;
		dscadsettings.range = (admode >> 3) & 0x1;
	}
	
	if( dscADSetSettings(dscb, &dscadsettings) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		sprintf(errorString, "Error in dscADSettings, DSC Error: #%d\n\t%s\n", errorParams.ErrCode, errorParams.errstring);
		errormessage(errorString);
		dscSetCalMux(dscb, 0);
		return 2;
	}

	//enable calmux
	dscSetCalMux(dscb, 1);

	dscDAConvert(dscb, 0, 0); //D/A output minimum voltage
	dscSleep(10);
	dscADSample(dscb, &adsampMin); 
	dscDAConvert(dscb, 0, 4095); //D/A output maximum voltage
	dscSleep(10);
	dscADSample(dscb, &adsampMax);

	*min = ADCodeToVolt(boardType, admode, adsampMin);
	*max = ADCodeToVolt(boardType, admode, adsampMax);
		
	dscSetCalMux(dscb, 0);

	return 0;

} // DAMinMaxVolt


/*===================================================================
	Prints out the millisecond time as long.
====================================================================*/

long ms()
{
	DWORD ms;
	dscGetTime(&ms);
	return ms;
}


/*===================================================================
	Gets the reference voltages and stores value in param reference
====================================================================*/

BYTE getReference(FLOAT * reference)
{
	BYTE result = 0; 
	if((result=referenceVoltages(0)) != DE_NONE)
		return result;

	for(int i=0; i < 5; i++)
		reference[i] = refs[i];

	return result;
}


/*===================================================================
	Finds the DMM-48-AT A/D mode.
	To find DMM-48-AT mode, the D/A is loop back to calmux channel 0. 
	D/A then output the D/A code 0 and 4095 and is sampled.
	Since D/A range is fixed to 0V to 4.096V, the sampled A/D code range 
	will determine the mode the DMM-48-AT is in.
====================================================================*/

BYTE getDMM48Mode()
{
	DSCSAMPLE sample, sample2;
	DSCADSETTINGS dscadsettings;
	dscadsettings.current_channel = 6; // D/A Loop back
	dscadsettings.load_cal = 0;
		

	// mode 8 is +-10V, mode 9 is +-5V, mode 13 is 0-5V
	if( dscSetCalMux(dscb, 1) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		printf("\n\t");
		dscSetCalMux(dscb, 0);
		printf("Error in dscSetCalMux, DSC Error: #%d\n\t%s\n",  errorParams.ErrCode, errorParams.errstring);
		sprintf(errorString, "Error in dscSetCalMux, DSC Error: #%d\n\t%s\n",  errorParams.ErrCode, errorParams.errstring);
		fileprint(errorString);
		endScreen();
	}

	if(dscADSetSettings(dscb, &dscadsettings) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		printf("\n\t");
		dscSetCalMux(dscb, 0);
		printf("Error in dscADSetSettings, DSC Error: #%d\n\t%s\n",  errorParams.ErrCode, errorParams.errstring);
		sprintf(errorString, "Error in dscADSetSettings, DSC Error: #%d\n\t%s\n",  errorParams.ErrCode, errorParams.errstring);
		fileprint(errorString);
		endScreen();
	}

	dscDAConvert(dscb, 0, 0);
	
	if(dscADSample(dscb, &sample) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		printf("\n\t");
		dscSetCalMux(dscb, 0);
		printf("Error in dscADSample, DSC Error: #%d\n\t%s\n",  errorParams.ErrCode, errorParams.errstring);
		sprintf(errorString, "Error in dscADSample, DSC Error: #%d\n\t%s\n",  errorParams.ErrCode, errorParams.errstring);
		fileprint(errorString);
		endScreen();
	}
	
	dscDAConvert(dscb, 0, 4095);
	
	if(dscADSample(dscb, &sample2) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		printf("\n\t");
		dscSetCalMux(dscb, 0);
		printf("Error in dscADSample, DSC Error: #%d\n\t%s\n",  errorParams.ErrCode, errorParams.errstring);
		sprintf(errorString, "Error in dscADSample, DSC Error: #%d\n\t%s\n",  errorParams.ErrCode, errorParams.errstring);
		fileprint(errorString);
		endScreen();
	}

	if( sample < -10000)
		DMM48Mode = 13; // 0-5V
	else if( sample2 > 20000)
		DMM48Mode = 9; // +-5V
	else 
		DMM48Mode = 8; // +-10V

	dscSetCalMux(dscb, 0);

	return DMM48Mode;
}


/*===================================================================
	Prints message to file
====================================================================*/

void fileprint(char* message)
{
	if( logBool)
		fprintf(logfile, message);
}


/*===================================================================
	Copies one string to another
====================================================================*/

void errormessage(char * message)
{
	strcpy(errorString, message);
}


/*========================================================
	Function reads in a string of input and returns
	the number of characters in the input. Returns 0 if the 
	first input from stin is a return character.
===========================================================*/

int optionInput(char * data)
{
	int start = 0;
	char key = 0;

	do
	{
		key = getchar();
		if( key =='\n' && start == 0)
			return 0;
		else if( key == '\n')
			break;
		
		data[start] = key;
		start ++;

	} while(1);

	return start;
}


/*===================================================================
	Sets all trimDAC values to param value.
====================================================================*/

BYTE SetAllTrimDAC(DSCB dscb, BYTE value)
{
	int i;
	
	for (i=0; i<8; i++)
	{
		if( dscSetTrimDac(dscb, i, value) != DE_NONE)	
		{
			dscGetLastError(&errorParams);
			sprintf(errorString, "Error in dscSetTrimDac, DSC Error: #%d\n\t%s\n",  errorParams.ErrCode, errorParams.errstring);
			errormessage(errorString);
			return 1;
		}
	}

	return 0;
}