//////////////////////////////////////////////////////////////////////////////
//	(c) Diamond Systems Corporation 2004
//	
//	http://www.diamondsystems.com
//
//	(c) Copyright 2004 Diamond Systems Corporation. Use of this source code
//	is subject to the terms of Diamond Systems' Software License Agreement.
//	Diamond Systems provides no warranty of proper performance if this
//	source code is modified.
//
//	By KL
//	Updated 1/15/04
///////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include "dscud.h"
#include "interrupt.h"

int numInterrupts;      //variable for TimerTest. Displays number of timers interrupts occured.


/*===================================================================
	Interrupt Quick Test. Performs A/D Scan interrupt on calmux channels
	and compares with reference voltages
====================================================================*/

BYTE IntTest(BYTE boardType, DSCB dscb)
{
	ERRPARAMS errorParams;  //structure with error information
	FLOAT refs[5];          //array for sotring reference voltages
	char errorString[500];  //error string
	
	int num_chan;   //number of channels
	int refIndex;   //index for refs array
	int tolerance;  //value tolerance
	int error;      //number of errors
	long diff;      //difference between expected and scanned values
	FLOAT hertz;    //rate of interrupt
	DSCSAMPLE adcode;
	BYTE result, admode;
	DSCAIOINT dscaioint;
	DSCADSETTINGS dscadsettings;
	DSCS dscs;


	admode = 8; //A/D settings mode. Mode 8 is +-10V.

	clrscr();

	printf("\n");
	printf("    *====================== Interrupt Test ======================*\n\n");
	fileprint("\n    *====================== Interrupt Test ======================*\n\n");

	getReference(refs);

	if( boardType == DSC_DMMAT)
		tolerance = 50;
	else if( (boardType == DSC_DMM16AT) || (boardType == DSC_DMM32) )
		tolerance = 200;
	else
		tolerance = 200;

	hertz = 200.0;


	printf("\t");
	printf("Running in A/D mode +- 10V, bipolar, gain of 1\n");
	printf("\t");
	printf("30 samples/6 scans at scan rate of %6.1f Hz per channel,\n", hertz); 
	printf("\t");
	printf("scanning cal mux channels\n\n");
	
	//setting up scan interrupt
	dscaioint.num_conversions = 30;
	dscaioint.conversion_rate = hertz;
	dscaioint.low_channel = 0;
	dscaioint.high_channel = 4;
	dscaioint.fifo_enab = 0;
	dscaioint.fifo_depth = 256;
	dscaioint.cycle = 0;
	dscaioint.internal_clock = 1;
	dscaioint.external_gate_enable = 0;
	dscaioint.internal_clock_gate = 0;
	dscaioint.dump_threshold = 0;

	num_chan = dscaioint.high_channel - dscaioint.low_channel + 1;

	//setting up A/D settings.
	dscadsettings.range = 1;
	dscadsettings.polarity = 0;
	dscadsettings.gain = 0;
	dscadsettings.load_cal = 1;
	dscadsettings.current_channel = 0;
	dscadsettings.scan_interval = SCAN_INTERVAL_5;
	
	if(boardType == DSC_DMM48)
	{
		//reset settings for DMM-48 after A/D mode call
		admode = getDMM48Mode();
		dscadsettings.gain = (admode & 0x3);
		dscadsettings.polarity = (admode >> 2) & 0x1;
		dscadsettings.range = (admode >> 3) & 0x1;
	}


	if( ( result = dscADSetSettings( dscb, &dscadsettings ) ) != DE_NONE )
	{
		dscSetCalMux(dscb, 0);
		dscGetLastError(&errorParams);
		sprintf(errorString, "dscADSetSettings Error, DSC Error: #%d, %s\n", result, errorParams.errstring);
		errormessage(errorString);
		return 1;
	}
	
	//enable calmux
	dscSetCalMux(dscb, 1);

	dscaioint.sample_values = (DSCSAMPLE*) malloc(sizeof(DSCSAMPLE) * dscaioint.num_conversions);
	memset(dscaioint.sample_values, 0, sizeof(DSCSAMPLE) * dscaioint.num_conversions);

	result = dscADSampleInt(dscb, &dscaioint);
	
	if( result != DE_NONE)
	{
		dscGetLastError(&errorParams);
		sprintf(errorString, "Error in dscADScanInt, DSC Error: #%d\n\t%s", result, errorParams.errstring);
		errormessage(errorString);
		dscSetCalMux(dscb, 0);
		free(dscaioint.sample_values);
		return 2;
	}

	dscs.transfers = 0;
	dscs.overflows = 0;
	dscs.op_type = OP_TYPE_INT;

	//scan interrupt should take less than 500 milliseconds to complete
	dscSleep(500);
	dscGetStatus(dscb, &dscs);
	
	//if no transfers have occured or there was an overflow
	if( dscs.overflows > 0  || dscs.transfers == 0)
	{
		if( dscs.overflows > 0)
			sprintf(errorString, "Interrupt Test Failed, fifo overflow");
		else
			sprintf(errorString, "Interrupt Test Failed, dscs.transfers == 0");

		errormessage(errorString);
		dscCancelOp(dscb);
		dscSetCalMux(dscb, 0);
		free(dscaioint.sample_values);
		return 3;
	}

	dscCancelOp(dscb);

	error = 0;

	printf("\t");
	printf("Sample#   Channel   Expected   Sampled   Difference   Pass/Fail\n");
	fileprint("\tSample#   Channel   Expected   Sampled   Difference   Pass/Fail\n");
			
	for( int i = 0; i < dscaioint.num_conversions; i ++)
	{
     	refIndex = (i%num_chan);
		adcode = voltToADCode(boardType, admode, refs[refIndex]);              //convert reference voltages to A/D code
		diff = labs(((long)adcode) - (DSCSAMPLE)(dscaioint.sample_values[i])); //find difference between scanned and expected

		if( ((i % 15) == 0) && (i != 0) )
		{
			continueScreen();
			printf("\t");
			printf("Sample#   Channel   Expected   Sampled   Difference   Pass/Fail\n");
			fileprint("\tSample#   Channel   Expected   Sampled   Difference   Pass/Fail\n");
		}

		printf("\t");
		printf("%7d   %7d   %8d   %7d   %10ld        ", i, (i%num_chan) + dscaioint.low_channel,
					adcode, (DSCSAMPLE)dscaioint.sample_values[i], diff);

		sprintf(errorString, "\t%7d   %7d   %8d   %7d   %10ld        ", i, (i%num_chan) + dscaioint.low_channel,
					adcode, (DSCSAMPLE)dscaioint.sample_values[i], diff);
		fileprint(errorString);

		if( diff > tolerance)
		{
			printf("Fail\n");
			fileprint("Fail\n");
			error ++;
		}
		else
		{
			printf("Pass\n");
			fileprint("Pass\n");
		}
	}
	
	free(dscaioint.sample_values);
	dscSetCalMux(dscb, 0);
	endScreen();
	return 0;
}


/*===================================================================
	Interrupt User Test. Allows users to setup their own interrupts
====================================================================*/

BYTE IntUserTest(BYTE boardType, DSCB dscb)
{
	ERRPARAMS errorParams;   //structure with error information
	char errorString[500];   //error string
	
	int option;              //determines which interrupt to perform, sample or scan
	int num_chan;            //number of channels
	int lowchan;             //low channel
	int highchan;            //high channel
	int temp;                //variable for scanf integers
	long templ;              //variable for scanf long
	float tempf;             //variable for scanf float
	DSCS dscs;  
	BYTE result;
	BYTE admode = 8;
	DSCAIOINT dscaioint;
	DSCADSETTINGS dscadsettings;
	float duration, voltage;

	//clear screen
	clrscr();

	printf("\n");
	printf("    *==================== Interrupt User Test ====================*\n\n");

	fileprint("\n    *==================== Interrupt User Test ====================*\n\n");

	printf("\t");
	printf("Running in A/D mode +- 10V, bipolar, gain of 1\n\n");

	//choose which interrupt to perform, sample or scan
	do{
		printf("\t");
		printf("Enter (0) for sampleint, (1) for scanint : ");
		scanf("%d", &option);
	}
	while( option != 0 && option != 1);


	//input number of converions and conversion rate
	do{
		printf("\t");
		printf("Enter number of conversions : ");
		scanf("%ld", &templ);
		printf("\t");
		dscaioint.num_conversions = (DWORD)templ;
		if( option == 0 )
			printf("Enter conversion rate : ");
		else 
			printf("Enter scan rate : ");
		scanf("%f", &tempf);
		dscaioint.conversion_rate = (FLOAT)tempf;

		if( dscaioint.num_conversions == 0 || dscaioint.conversion_rate == 0)
			printf("\n\tNum conversions and/or conversion rate can't be 0.\n\n");
		else
			break;
	}
	while(true);
	
	//input low and high channel
	do
	{
		printf("\t");
		printf("Enter low channel : ");
		scanf("%d", &lowchan);
		dscaioint.low_channel = (BYTE)lowchan;
		printf("\t");
		printf("Enter high channel : ");
		scanf("%d", &highchan);
		dscaioint.high_channel = (BYTE)highchan;

		if(dscaioint.num_conversions%(highchan - lowchan + 1) != 0)
			printf("\n\tNumber of channels must be a factor of number of conversions.\n\n");
		else
			break;
	}
	while(true);
	

	//input fifo depth
	do
	{		
		printf("\t");
		printf("Enter (0) to disable fifo, (1) to enable fifo : ");
		scanf("%d", &temp);
		dscaioint.fifo_enab = (BYTE)temp;

		getchar();

		if( (boardType == DSC_DMM32 || boardType == DSC_HERCEBX) && dscaioint.fifo_enab)
		{
			printf("\tEnter fifo depth : ");
			scanf("%d", &temp);
			dscaioint.fifo_depth = (WORD) temp;
			getchar();
		}
		else
			dscaioint.fifo_depth = 256;

		if(dscaioint.fifo_enab && dscaioint.fifo_depth == 0)
			printf("\n\tFIFO depth has to be a factor of num conversions\n\n");
		else if(dscaioint.fifo_enab && dscaioint.num_conversions%temp != 0)
			printf("\n\tFIFO depth has to be a factor of num conversions\n\n");
		else
			break;
	}
	while(true);

	//setup interrupts
	dscaioint.cycle = 0;
	dscaioint.internal_clock = 1;
	dscaioint.external_gate_enable = 0;
	dscaioint.internal_clock_gate = 0;
	dscaioint.dump_threshold = 0;

	num_chan = dscaioint.high_channel - dscaioint.low_channel + 1;

	//setup A/D settings
	dscadsettings.range = 1;
	dscadsettings.polarity = 0;
	dscadsettings.gain = 0;
	dscadsettings.load_cal = 1;
	dscadsettings.current_channel = 0;
	dscadsettings.scan_interval = SCAN_INTERVAL_5;

	if(boardType == DSC_DMM48)
	{
		//setup A/D settings for DMM-48
		admode = getDMM48Mode();
		dscadsettings.gain = (admode & 0x3);
		dscadsettings.polarity = (admode >> 2) & 0x1;
		dscadsettings.range = (admode >> 3) & 0x1;
	}

	if( ( result = dscADSetSettings( dscb, &dscadsettings ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		sprintf(errorString, "Error in dscADSetSettings, DSC Error: #%d, %s\n", result, errorParams.errstring);
		errormessage(errorString);
		return 2;
	}

	//alllocate memory for buffer
	dscaioint.sample_values = (DSCSAMPLE*) malloc(sizeof(DSCSAMPLE) * dscaioint.num_conversions);
	memset(dscaioint.sample_values, 0, sizeof(DSCSAMPLE) * dscaioint.num_conversions);

	//call dscADSampleInt or dscADScanInt
	if( option == 1)
	{
		duration = (float)dscaioint.num_conversions / ((float)num_chan * (float)dscaioint.conversion_rate);
		result = dscADScanInt(dscb, &dscaioint);
	}
	else
	{
		duration = (float)dscaioint.num_conversions / (float)dscaioint.conversion_rate;
		result = dscADSampleInt(dscb, &dscaioint);
	}

	if( result != DE_NONE)
	{
		dscGetLastError(&errorParams);
		sprintf(errorString, "Error in dscADSampleInt/ScanInt, DSC Error: #%d, %s\n", result, errorParams.errstring);
		errormessage(errorString);
		free(dscaioint.sample_values);
		return 3;
	}

	printf("\n\tGenerating Conversions for %5.2f sec\n\n", duration);
	sprintf(errorString, "\n\tGenerating Conversions for %5.2f sec\n\n", duration);
	fileprint(errorString);
	
	dscs.transfers = 0;
	dscs.overflows = 0;
	dscs.op_type = OP_TYPE_INT;

	//wait for interrupt to finish
	while( !kbhit() && dscs.op_type != OP_TYPE_NONE)
	{
		dscGetStatus(dscb, &dscs);
		dscSleep(100);
		if( dscs.overflows > 0 )
		{
			sprintf(errorString, "Interrupt User Test Failed, fifo overflow\n");
			errormessage(errorString);
			dscCancelOp(dscb);
			free(dscaioint.sample_values);
			return 4;
		}
	}

	dscCancelOp(dscb);

	printf("\t");
	printf("Sample#   Channel#   Sampled Value   A/D Voltage\n");
	fileprint("\tSample#   Channel#   Sampled Value   A/D Voltage\n");

	for( int i = 0; i < dscaioint.num_conversions; i ++)
	{	
		//convert A/D sampled values to voltage
		voltage = ADCodeToVolt(boardType, admode, dscaioint.sample_values[i]);

		printf("\t");
		printf("%7d   %8d   %13d   %11.3f\n", i, (i%num_chan) + dscaioint.low_channel, dscaioint.sample_values[i], voltage);
		sprintf(errorString, "\t%7d   %8d   %13d   %11.3f\n", i, (i%num_chan) + dscaioint.low_channel, dscaioint.sample_values[i], voltage);
		fileprint(errorString);

		if( ((i % 15) == 0) && (i != 0))
		{
			if( quitScreen() == 1)
				break;

			printf("\n\t");
			printf("Sample#   Channel#   Sampled Value   A/D Voltage\n");
			fileprint("\tSample#   Channel#   Sampled Value   A/D Voltage\n");
		}
	}

	free(dscaioint.sample_values);
	endScreen();
	return 0;
}


/*===================================================================
	Timer Test. Tests counter timer clock
====================================================================*/

BYTE TimerTest(BYTE boardType, DSCB dscb)
{
	ERRPARAMS errorParams;  //structure with error information
	char errorString[500];  //error string
	
	DSCUSERINT dscuserint;
	BYTE result;
	int seconds;        //duration of interrupt test
	int ctr1IntRate;    //counter rate 
	long end;           //end time
	long start;         //start time

	clrscr();

  	dscCancelOp(dscb);

	ctr1IntRate = 150;
	numInterrupts = 0;
	seconds = 3;
	start = 0;
	end = 0;

	printf("\n");
	printf("    *================= Timer Interrupt Test =================*\n\n");
	fileprint("\n    *================= Timer Interrupt Test =================*\n");

	printf("\t");
	printf("Running Counter Timer for %d seconds\n", seconds);

	//setup and start user/counter interrupt
	dscuserint.intsource = 0;
	dscuserint.counter = 0;
	dscuserint.clksource = 0; // 10 MHz clock
	dscuserint.rate = ctr1IntRate;
	result = dscUserInt(dscb, &dscuserint, (DSCUserInterruptFunction)UserFunc);

	if( result != DE_NONE)
	{
		dscGetLastError(&errorParams);
		sprintf(errorString, "Error in dscUserInt, DSC Error: #%d\n\t%s", result, errorParams.errstring);
		errormessage(errorString);
		dscCancelOp(dscb);
		dscClearUserInterruptFunction(dscb);
		return 1;
	}

	//run interrupt for a certain duration
	start = ms();
	end = ms();
	while( labs(end - start) < ( seconds * 1000))
	{
		end = clock();
	}

	dscCancelOp(dscb);

	if( abs(numInterrupts - (ctr1IntRate * seconds)) > (ctr1IntRate * seconds) / 50)
	{
		sprintf(errorString, "Timer Test Failed, expected # interrupts %d, gotten %d", (ctr1IntRate * seconds), numInterrupts);
		errormessage(errorString);
		dscCancelOp(dscb);
		dscClearUserInterruptFunction(dscb);
		return 2;
	}

	printf("\n\tTimer Test Passed\n");
	fileprint("\n\tTimer Test Passed\n");
	dscCancelOp(dscb);
	dscClearUserInterruptFunction(dscb);
	endScreen();
	return 0;
}


//user function
void UserFunc()
{
	numInterrupts++;	
}




