//EEPDISP program
//to display AT board EEPROMS
// dev by KL

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "dscud.h"

#define REF_BASE 32

DSCB board;

int key;

DFLOAT BytesToRefVolt(LONG bytes)
{
	LONG mybyte[3]; 

	DFLOAT retval;
	
	mybyte[2] = (bytes >> 16) & 0xFF;
	mybyte[1] = (bytes >> 8) & 0xFF; 
	mybyte[0] = bytes & 0xFF;

	retval = ((mybyte[2] & 0x7E) >> 1) + ( ( ( (mybyte[2] & 1) << 16) + (mybyte[1] << 8) + mybyte[0]) / (DFLOAT)100000.0 );

	if (mybyte[2] & 128) return (retval * -1);
	return retval;
} 

void displayEEPROMInfo(int d)
{
	printf("EEPROM Contents:\n\n");
	BYTE value;

	for (int x = 0; x < 128; x++)
	{
		dscGetEEPROM(board, x, &value);

		if (x % 8 == 0) printf("%03d:  ",x);
		if (d) 
			printf("%02X ",value);
		else 
			printf("%03d ",value);

		if ((x+1) % 4 == 0 && !(x % 8 == 0)) printf("   ");

		if ((x+1) % 8 == 0) printf("\n");
	}

	printf("\nPress c to continue\n");

	while( getchar() != 'c');{}
}

void displayTrimDACInfo()
{
	printf("A/D Calibration Settings:\n\n");
	BYTE values[4];

	printf("  Mode | TrimDAC 0 | TrimDAC 1 | TrimDAC 2 | TrimDAC 3\n");
	printf("-------+-----------+-----------+-----------+------------\n");

	for (int x = 0; x < 4; x++)
	{
		dscGetEEPROM(board, x, &(values[x]));
	}

	printf("  Boot |  %02X (%03d) |  %02X (%03d) |  %02X (%03d) |  %02X (%03d)\n",values[0],values[0],values[1],values[1],values[2],values[2],values[3],values[3]);

	for (int m = 0; m < 16; m++)
	{
		for (int i = 0; i < 4; i++)
		{
			dscGetEEPROM(board, 64 + (m * 4) + i, &(values[i]));
		}

		printf("   %02d  |  %02X (%03d) |  %02X (%03d) |  %02X (%03d) |  %02X (%03d)\n",m,values[0],values[0],values[1],values[1],values[2],values[2],values[3],values[3]);
	}

	printf("\nPress c to continue\n");

	while( getchar() != 'c');{}

}

int matchfour(BYTE* m1, BYTE* m2)
{
	if (m1[0] == m2[0] && m1[1] == m2[1] && m1[2] == m2[2] && m1[3] == m2[3] ) return 1;
	else return 0;
}

void setBootMode()
{
	int dial[4] = { '|', '/', '-', '\\' };
	int counter = 0;

	BYTE boot[4];
	BYTE cur[4];
	
	int curmode = -1;
	int newmode = -1;

	printf("Current boot mode:  ");

	for (int b = 0; b < 4; b++) { dscGetEEPROM(board, b, &(boot[b])); printf("\b%c",dial[counter%4]); counter++; }

	for (int m = 0; m < 16; m++)
	{
		for (int i = 0; i < 4; i++) { dscGetEEPROM(board, 64 + (m * 4) + i, &(cur[i])); printf("\b%c",dial[counter%4]); counter++; }

		if (matchfour(cur,boot))
		{
			curmode = m;
			m = 16;
		}
	}

	if (curmode != -1) 
		printf("\b%d\n\n",curmode);
	else
		printf("\bUndefined\n\n");

	
	while (newmode < 0 || newmode > 15)
	{
		printf("Enter new boot mode: ");
		scanf("%d",&newmode);

		if (newmode < 0 || newmode > 15)
			printf("Invalid Mode (must be 0-15)\n");
	}

	for (int c = 0; c < 4; c++)
	{
		BYTE value;
		dscGetEEPROM(board, 64 + (newmode * 4) + c, &value);
		dscSetEEPROM(board, c, value);
	}

	printf("\nDone\n\n");
}

void referenceVoltages()
{
	printf("\nCurrent Reference Voltages:\n\n");

	LONG data;
	LONG b1,b2,b3;
	BYTE b4,b5,b6;

	for (int x = 0; x < 6; x++)
	{
		dscGetEEPROM(board, REF_BASE + (x * 3)    , &b4);
		dscGetEEPROM(board, REF_BASE + (x * 3) + 1, &b5);
		dscGetEEPROM(board, REF_BASE + (x * 3) + 2, &b6);

		b1 = b4;
		b2 = b5;
		b3 = b6;

		data = (b1 << 16) | (b2 << 8) | b3;

		printf("Reference %d: %9.2f\n",x,BytesToRefVolt(data));
	}
}

void beginMain()
{
	int quit = 0;
	while (!quit)
	{
		printf("\nMenu:\n");
		printf("1) Dump EEPROM Contents in HEX\n");
		printf("2) Dump EEPROM Contents in DECIMAL\n");
		printf("3) View A/D Calibration settings\n");
		printf("4) Get/Set Boot Mode\n");
		printf("5) Reference Voltages\n"); 
		printf("6) Quit\n");
		printf(": ");

		key = getchar();

		printf("%c\n\n",key);

		if (key == '1') displayEEPROMInfo(1);
		if (key == '2') displayEEPROMInfo(0);
		if (key == '3') displayTrimDACInfo();
		if (key == '4') setBootMode();
		if (key == '5') referenceVoltages();
		if (key == '6' || key == 'q') quit = 1;

	}
		
}


void main()
{
	int init = dscInit(DSC_VERSION);

	if (init) 
	{
		printf("Could not initialize DSCUD\n");
		exit(1);
	}

	int boardmacro = -1;

	while (boardmacro == -1)
	{
		printf("Choose board type: \n");
		printf("1) DMM-AT\n");
		printf("2) DMM-16-AT\n");
		printf("3) DMM-32-AT\n");
		printf(": ");

		key = getchar();
		printf("%c\n\n",key);

		if (key == '1') boardmacro = DSC_DMMAT;
		if (key == '2') boardmacro = DSC_DMM16AT;
		if (key == '3') boardmacro = DSC_DMM32;
	}

	int baseaddress = 0;

	printf("Enter the base address (e.g. \"0x300\"): ");
	scanf("%x",&baseaddress);

	DSCCB dsccb;
	dsccb.base_address = baseaddress;
	dsccb.int_level = 5;
	dsccb.dma_level = 1;
	dsccb.clock_freq = 10000000;

	int binit = dscInitBoard(boardmacro, &dsccb, &board);

	if (binit)
	{
		printf("Could not initialize the board\n");
		exit(2);
	}

//	dscSetEEPROM(board, 6, 0xff);

	beginMain();

	dscFreeBoard(board);
	dscFree();
}
