#include <afxdlgs.h>
#include <commctrl.h>
#include <windowsx.h>
#include <winuser.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <dos.h>
#include <conio.h>
#include <windows.h>
#include "resource.h"
#include "dscud.h"

#define INPORT 1
#define OUTPORT 0
#define DIOCONFIG 0x02

char buf[10000];

HWND hMain;

DSCB board;
DSCCB settings;
DSCSAMPLE samples[16];
DSCDACODE output[4];
DSCDACS dscdacs;
DSCADSCAN dscadscan;
DSCADSETTINGS dscadset;

HICON offon[2];

HWND progressbars[16];
HWND advalues[16];
HWND volts[16];
HWND dins[8];
HWND douts[8];

int gain = 1;

int counter = 0;

//int channeldispatch[16] = { 0, 8, 1, 9, 2, 10, 3, 11, 4, 12, 5, 13, 6, 14, 7, 15 };
//int channeldispatch[16] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15 };
int channeldispatch[16] = { 0, 2, 4, 6, 8, 10, 12, 15, 1, 3, 5, 7, 9, 11, 13, 14 };

float admax = (float)10.0;
float admin = (float)-10.0;
float adran = (float)20.0;
float addiv = (float)20.0/(float)65536.0;
float damax = (float)5.0;
float damin = (float)-5.0;
float daran = (float)10.0;
float dadiv = (float)10.0/(float)4096.0;

float adcodeToVolt(DSCSAMPLE code)
{
	FLOAT ctmp = (FLOAT)code + (FLOAT)32768.0;
	//ctmp = (FLOAT)ctmp / (FLOAT)65536.0;
	ctmp *= addiv;
	ctmp += admin;
	return ctmp;
}

float dacodeToVolt(DSCDACODE code)
{
	FLOAT ctmp = (FLOAT)code;// + (FLOAT)2048.0;
	//ctmp = (FLOAT)ctmp / (FLOAT)65536.0;
	ctmp *= dadiv;
	ctmp += damin;
	return ctmp;
}

void loadConfig()
{
	FILE* file = fopen("prom.cfg","rb");

	if (file != NULL)
	{
		fscanf(file, "ADMAX=%f\r\n",&admax);
		fscanf(file, "ADMIN=%f\r\n",&admin);
		fscanf(file, "DAMAX=%f\r\n",&damax);
		fscanf(file, "DAMIN=%f\r\n",&damin);
		fclose(file);

		adran = admax - admin;
		addiv = adran / (float)65536.0;

		daran = damax - damin;
		dadiv = daran / (float)4096.0;
	}
}

int CALLBACK DialogProc (HWND hwnd, UINT message,WPARAM wParam,LPARAM lParam)
{
	switch (message)
	{
	case WM_CLOSE:
		EndDialog(hMain,0);
		dscFree();
		exit(0);
		break;

	case WM_COMMAND:
	
		{
			BYTE outbyte = 0;
			for (int x = 0; x < 8; x++)
			{
				outbyte |= Button_GetCheck(douts[7-x]) << x;
			}
			dscDIOOutputByte(board, OUTPORT, outbyte);
			
		}

		break;

	case WM_VSCROLL:

		for (int x = 0; x < 4; x++)
		{
			output[x] = 4095 - SendMessage(GetDlgItem(hMain,IDC_SLIDER1+x), TBM_GETPOS, 0, 0);
			sprintf(buf,"%1.2f",dacodeToVolt(output[x]));
			Static_SetText(GetDlgItem(hMain,IDC_DAVALUE1+x),buf);
		}
		
		dscDAConvertScan(board, &dscdacs);

		break;

	}

	return 0;
}


LRESULT CALLBACK WndProc(HWND hWnd, unsigned int iMessage, WPARAM wParam, LPARAM lParam)
{
	switch (iMessage)
	{
	case WM_CLOSE:
		dscFree();
		exit(0);
		break;
	}
	return DefWindowProc(hWnd, iMessage, wParam, lParam);
}


int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	HFONT font = CreateFont(24,8,0,0,12,0,0,0,0,0,0,0,0,"Impact"); 
	
	dscdacs.output_codes = output;
	dscadscan.low_channel = 0;
	dscadscan.high_channel = 15;

	dscadset.gain = 0;
	dscadset.current_channel = 0;
	dscadset.load_cal = 0;

	AfxInitRichEdit();

	WNDCLASS WndClass;

	memset(&WndClass, 0, sizeof(WndClass));
	WndClass.cbClsExtra = 0;
	WndClass.cbWndExtra = 0;
	
	WndClass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
	WndClass.hCursor = LoadCursor(NULL, IDC_ARROW);
	WndClass.hIcon = LoadIcon(hInstance, NULL);
	WndClass.hInstance = hInstance;
	WndClass.lpfnWndProc = WndProc;
	WndClass.lpszClassName = "Promdemo";
	WndClass.style = CS_HREDRAW | CS_VREDRAW;
	
	if(!RegisterClass(&WndClass)) return 0;


	loadConfig();

	int error = dscInit(DSC_VERSION);
	if (error != DE_NONE)
	{
		sprintf(buf,"dscInit returned with error code %d",error);
		MessageBox(NULL,buf,"dscInit error",MB_OK);
	}

	settings.base_address = 0x280;
	settings.int_level = 5;
	settings.dma_level = 1;

	error = dscInitBoard(DSC_PROM, &settings, &board);
	if (error != DE_NONE)
	{
		sprintf(buf,"dscInitBoard returned with error code %d",error);
		MessageBox(NULL,buf,"dscInitBoard error",MB_OK);
	}

	hMain = CreateDialog(hInstance,MAKEINTRESOURCE(IDD_PROMDEMO),NULL,DialogProc);

	offon[0] = (HICON)LoadImage(hInstance,MAKEINTRESOURCE(IDI_OFF),IMAGE_ICON,16,16,0);
	offon[1] = (HICON)LoadImage(hInstance,MAKEINTRESOURCE(IDI_ON),IMAGE_ICON,16,16,0);

	SendMessage(GetDlgItem(hMain,IDC_TITLE), WM_SETFONT, (WPARAM)font, 1);

	dscADSetSettings(board, &dscadset);

	BYTE config = DIOCONFIG;
	dscDIOOutputByte(board, INPORT, 0);
	dscDIOSetConfig(board, &config);
	dscDIOOutputByte(board, OUTPORT, 0);
	
	for (int h = 0; h < 4; h++) { output[h] = 2048; dscdacs.channel_enable[h] = 1; }
	dscDAConvertScan(board, &dscdacs);

	for (int i = 0; i < 16; i++)
	{
		//set range for all ad meters
		SendMessage(GetDlgItem(hMain,IDC_PROGRESS1+i), PBM_SETRANGE, 0, MAKELPARAM(0,65535));
		//set meter for middle
		SendMessage(GetDlgItem(hMain,IDC_PROGRESS1+i), PBM_SETPOS, (WPARAM)32768, 0);
		//set ad text to 0
		Static_SetText(GetDlgItem(hMain,IDC_ADVALUE1+i), "0");
		//set volt value  to 0.0
		Static_SetText(GetDlgItem(hMain,IDC_VOLT1+i), "0.000");

		progressbars[i] = GetDlgItem(hMain,IDC_PROGRESS1+i);
		advalues[i] = GetDlgItem(hMain,IDC_ADVALUE1+i);
		volts[i] = GetDlgItem(hMain,IDC_VOLT1+i);

	}

	for (int j = 0; j < 4; j++)
	{
		//set range for sliders
		SendMessage(GetDlgItem(hMain,IDC_SLIDER1+j), TBM_SETRANGE, 1, MAKELPARAM(0,4095));
		//set position to middle
		SendMessage(GetDlgItem(hMain,IDC_SLIDER1+j), TBM_SETPOS, 1, (LPARAM)2047);

		sprintf(buf,"%1.2f",dacodeToVolt(2048));
		Static_SetText(GetDlgItem(hMain,IDC_DAVALUE1+j),buf);

		dscdacs.channel_enable[j] = 1;
	}

	for (int k = 0; k < 8; k++)
	{
		dins[k] = GetDlgItem(hMain,IDC_DIN1+k);
		douts[k] = GetDlgItem(hMain,IDC_DOUT1+k);

		SendMessage(dins[k], STM_SETICON, (WPARAM)offon[0], 0);
	}

	MSG Message;

	while (1)
	{
		if (PeekMessage(&Message, hMain, 0, 0, PM_REMOVE))
		{
			TranslateMessage(&Message);
			DispatchMessage(&Message);
		}		

		//update A/D
		dscADScan(board, &dscadscan, samples);
		/*for (int c = 0; c < 16; c++)
		{
			dscADSetChannel(board,c,c);
			dscADSample(board,&(samples[c]));
		}*/

		for (int x = 0; x < 16; x++)
		{
			while (PeekMessage(&Message, hMain, 0, 0, PM_REMOVE))
			{
				TranslateMessage(&Message);
				DispatchMessage(&Message);
			}	

			sprintf(buf,"%d",samples[x]);
			Static_SetText(advalues[channeldispatch[x]], buf);
			
			SendMessage(progressbars[channeldispatch[x]], PBM_SETPOS, (WPARAM)(samples[x] + 32768), 0);

			sprintf(buf,"%4.3f",adcodeToVolt(samples[x]));
			Static_SetText(volts[channeldispatch[x]], buf);
		}

		BYTE value;
		dscDIOInputByte(board, INPORT, &value);

		for (int y = 0; y < 8; y++)
		{
			while (PeekMessage(&Message, hMain, 0, 0, PM_REMOVE))
			{
				TranslateMessage(&Message);
				DispatchMessage(&Message);
			}	

			if (value & (1<<y))
				SendMessage(dins[7-y],STM_SETICON, (WPARAM)offon[1], 0);
			else
				SendMessage(dins[7-y],STM_SETICON, (WPARAM)offon[0], 0);
		}
	}

	return 0;
}